procedure dkcor48 (hdrcom, image, darktime, dark, dmean, logfile, darklog,
		verbose, first)

string	hdrcom	=""		{prompt="Header controled command"}
string	image	=""		{prompt="Image name"}
real	darktime=INDEF		{prompt="Dark time of image"}
string	dark	=""		{prompt="Dark current calibration image"}
bool	dmean	=yes		{prompt="Use mean instead of dark image"}
string	logfile	=""		{prompt="Text logfile ccd48proc"}
string	darklog	=""		{prompt="Text logfile dark correction"}
bool	verbose	=yes		{prompt="Verbose messages?"}
bool	first	=no		{prompt="First image in list?"}

# @(#) subtask dkcor48            Author: R.A. Jansen -- January 17 1996
# @(#) 
# @(#) subtask called from  'ccd48proc'  to subtract a possible non-zero
# @(#) dark current.
# @(#)
# @(#) Nov  3 2002 -- Headers are now updated  using  'artdata.mkheader'
# @(#)   rather than 'ccdhedit'.  Keywords are now 'DARKCOR' (with value
# @(#)   'PERFORM','OMIT', or 'COMPLETE'), 'DKCRIMAG' with image name as
# @(#)   its value and 'DKCRDATE' (FITS Y2K compliant date+time string).
# @(#)   Also, the mean dark level is logged to keyword 'DKCRMEAN'. RAJ.
# @(#) Aug 14 2003 -- Added 'hdrcom' to support header keyword controled
# @(#)   data reduction a la 'calstis', and added 'verbose' switch. RAJ.
# @(#) Nov 24 2003 -- Modified scaling of SUPERDARK image by  'darktime'
# @(#)   rather than 'exptime'. Typically, 'darktime=1.' for a SUPERDARK
# @(#)   image. RAJ.
# @(#) Jul  1 2004 -- Changed the decision criteria  for whether to take
# @(#)   any action  by checking also the maximum scaled dark current in
# @(#)   the supplied dark image (action if dc_max>0.5 ADU).  RAJ+AJM.

begin
	string	cval, cimg, dimg, cstr
	real	rlo, rhi, mean, dc, dcmax, tdrk, dtdrk, ratio

	# Initialize variables...
	cval  = hdrcom
	cimg  = image
	dimg  = dark	
	tdrk  = darktime
	dtdrk = 1.
	if ( darklog == "" ) darklog = "darkproc.log"

	# Clean-up any relevant debris...
	if ( access("dkcor.head") ) delete("dkcor.head", yes, verify-)

	# If first image in list, open darklog and start logging...
	if (first) {
	   sysinfo() ; gdate(utc=yes)
	   if ( access(darklog) ) {
	      print ("\n## CCD48PROC (DKCOR48)  "//sysinfo.sysmsg//"  "//gdate.tdate//"\n", >> darklog)
	   } else {
	      print ("## CCD48PROC (DKCOR48)  "//sysinfo.sysmsg//"  "//gdate.tdate//"\n", >> darklog)
	   }
	}

	# Check whether image was processed already...
	if ( cval == "COMPLETE" ) {
	   if (verbose) print ("    DARKCOR:  image was processed already; no action taken")
	   print ("  DARKCOR = \"COMPLETE\"; no action taken", >> logfile)
	   print (cimg//": DARKCOR = \"COMPLETE\"; no action taken", >> darklog)

	} else if ( cval == "OMIT" ) {
	   if (verbose) print ("    DARKCOR = \"OMIT\"; no action taken")
	   print ("  DARKCOR = \"OMIT\"; no action taken", >> logfile)
	   print (cimg//": DARKCOR = \"OMIT\"; no action taken", >> darklog)

	   # Add header keyword DARKCOR...
	   gdate(utc=yes)
	   hedit (cimg, "darkcor", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "cc48date", "", add-, del+, ver-, show-, upd+)
	   imheader (cimg, longheader+, > "dkcor.head")
	   printf("DARKCOR = %-21s / Dark image correction flag\n", "'OMIT'", >> "dkcor.head")
	   print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "dkcor.head")
	   mkheader (cimg, "dkcor.head", append-, verbose-)
	   delete ("dkcor.head", yes, verify-)

	} else {
	   if (verbose) printf("    DARKCOR ...")

	   # measure mean dark level in the dark image...
	   imgstat (dimg, lower=INDEF, upper=INDEF, binwidth=0.1, verbose-)
	   rlo = imgstat.midpt - 4*imgstat.stddev
	   rhi = imgstat.midpt + 4*imgstat.stddev
	   imgstat (dimg, lower=rlo, upper=rhi, binwidth=0.1, verbose-)
	   dc   = imgstat.mean
	   dcmax= imgstat.max
	   mean = dc

	   # compute scaled dark level...
	   hselect (dimg, "darktime", "yes" ) | scan (dtdrk)
	   dc    = (tdrk/dtdrk)*dc
	   dcmax = (tdrk/dtdrk)*dcmax

	   if ( dc < 0.1 && dcmax < 0.5 ) {
	      if (verbose) printf("  dc = %7f < 0.1 ADU (dc_max = %7f < 0.5 ADU); no action taken\n", dc, dcmax)
	      
	      # Add keywords DARKCOR, DKCRIMAG, DKCRMEAN and DKCRDATE...
	      gdate(utc=yes)
	      hedit (cimg, "darkcor", "", add-, del+, ver-, show-, upd+)
	      hedit (cimg, "dkcrimag", "", add-, del+, ver-, show-, upd+)
	      hedit (cimg, "dkcrmean", "", add-, del+, ver-, show-, upd+)
	      hedit (cimg, "dkcrdate", "", add-, del+, ver-, show-, upd+)
	      hedit (cimg, "cc48date", "", add-, del+, ver-, show-, upd+)
	      imheader (cimg, longheader+, > "dkcor.head")
	      printf("DARKCOR = %-21s / Dark image correction flag\n", "'OMIT'", >> "dkcor.head")
	      if ( strlen(dimg) <= 21 ) {
		 cstr = "'"//dimg//"'"
		 printf("DKCRIMAG= %-21s / Name of dark image\n", cstr, >> "dkcor.head")
	      } else {
		 if ( strlen(dimg) <= 47 ) {
		    cstr = "'"//dimg//"'"
		 } else {
		    cstr = "'..."//substr(dimg,(strlen(dimg)-43),strlen(dimg))//"'"
		 }
		 printf("DKCRIMAG= %-"//strlen(cstr)//"s / Name of dark image\n", cstr, >> "dkcor.head")
	      }
	      printf("DKCRMEAN= %20.2f  / Mean dark level (ADU) in image < 0.1\n", dc, >> "dkcor.head")
	      printf("DKCRMAX = %20.2f  / Max dark level (ADU) in image < 0.5\n", dcmax, >> "dkcor.head")
	      print ("DKCRDATE= '"//gdate.fdate//"' / Measurement mean dark level done", >> "dkcor.head")
	      print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "dkcor.head")
	      mkheader (cimg, "dkcor.head", append-, verbose-)
	      delete ("dkcor.head", yes, verify-)

	      # Log action to text log files...
	      printf("  DARKCOR:  "//gdate.fdate//" No action taken: mean=%7f of "//cstr//" < 0.1 ADU\n", dc, >> logfile)
	      printf(cimg//": DARKCOR:  "//gdate.fdate//" No action taken: mean=%7f of "//cstr//" < 0.1 ADU\n", dc, >> darklog)

	   } else {
	      # Subtract the scaled mean level or scaled dark image itself...
	      if (dmean) {
		 imarith (cimg, "-", dc, cimg, title="", divzero=0.,
			hparams="", pixtype="real", calctype="real",
			verbose+, noact-, >> darklog)
	      } else {
		 imarith (dimg, "*", (tdrk/dtdrk), "tmpdrk.fits", title="",
			divzero=0., hparams="", pixtype="real", verbose+,
			noact-, >> darklog)
		 imarith (cimg, "-", "tmpdrk.fits", cimg, title="",
			divzero=0., hparams="", pixtype="real", verbose+,
			noact-, >> darklog)
		 delete ("tmpdrk.fits", yes, verify-)
	      }

	      # Add keywords DARKCOR, DKCRIMAG, DKCRMEAN and DKCRDATE...
	      gdate(utc=yes)
	      hedit (cimg, "darkcor", "", add-, del+, ver-, show-, upd+)
	      hedit (cimg, "dkcrimag", "", add-, del+, ver-, show-, upd+)
	      hedit (cimg, "dkcrmean", "", add-, del+, ver-, show-, upd+)
	      hedit (cimg, "dkcrdate", "", add-, del+, ver-, show-, upd+)
	      hedit (cimg, "cc48date", "", add-, del+, ver-, show-, upd+)
	      imheader (cimg, longheader+, > "dkcor.head")
	      printf("DARKCOR = %-21s / Dark image correction flag\n", "'COMPLETE'", >> "dkcor.head")
	      if ( strlen(dimg) <= 21 ) {
		 cstr = "'"//dimg//"'"
		 printf("DKCRIMAG= %-21s / Name of dark image\n", cstr, >> "dkcor.head")
	      } else {
		 if ( strlen(dimg) <= 47 ) {
		    cstr = "'"//dimg//"'"
		 } else {
		    cstr = "'..."//substr(dimg,(strlen(dimg)-43),strlen(dimg))//"'"
		 }
		 printf("DKCRIMAG= %-"//strlen(cstr)//"s / Name of dark image\n", cstr, >> "dkcor.head")
	      }
	      if (dmean) {
	         printf("DKCRMEAN= %20.2f  / Mean dark level (ADU) subtracted\n", dc, >> "dkcor.head")
	         printf("DKCRMAX = %20.2f  / Max dark level (ADU) (ignored)\n", dcmax, >> "dkcor.head")
	         print ("DKCRDATE= '"//gdate.fdate//"' / Removal dark level using mean done", >> "dkcor.head")
	      } else {
	         printf("DKCRMEAN= %20.2f  / Mean dark level (ADU) in image\n", dc, >> "dkcor.head")
	         printf("DKCRMAX = %20.2f  / Max dark level (ADU) in image\n", dcmax, >> "dkcor.head")
	         print ("DKCRDATE= '"//gdate.fdate//"' / Removal dark level using image done", >> "dkcor.head")
	      }
	      print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "dkcor.head")
	      mkheader (cimg, "dkcor.head", append-, verbose-)
	      delete ("dkcor.head", yes, verify-)
	      
	      if (verbose) printf("  done (dc= %7f ADU)\n", dc)

	      # Log action to text log files...
	      print (cimg//":", >> darklog)
	      if (dmean) {
		 printf("  DARKCOR:  "//gdate.fdate//" Subtracted dark level using mean=%7f of "//cstr//"\n", dc, >> logfile)
		 print ("  DARKCOR:  "//gdate.fdate//" Subtracted dark level using mean of "//cstr, >> darklog)
		 printf("  DARKCOR:  "//gdate.fdate//" Maximum dark level %7f of "//cstr//" was ignored\n", dcmax, >> logfile)
		 print ("  DARKCOR:  "//gdate.fdate//" Maximum dark level of "//cstr//" was ignored", >> darklog)
	      } else {
		 printf("  DARKCOR:  "//gdate.fdate//" Subtracted dark level using "//cstr//": dc_mean = %7f, dc_max = %7f\n", dc, dcmax, >> logfile)
		 print ("  DARKCOR:  "//gdate.fdate//" Subtracted dark level using "//cstr, >> darklog)
	      }
	      printf("                                Mean dark current in "//cimg//" = %7f\n", dc, >> darklog)

	   }

	}

end
