procedure fixpx48 (hdrcom, image, fixfile, logfile, verbose)

string	hdrcom	=""		{prompt="Header controled command"}
string	image	=""		{prompt="Image name"}
string	fixfile	=""		{prompt="File listing bad lines and columns"}
string	logfile	=""		{prompt="Text logfile"}
bool	verbose =yes		{prompt="Verbose messages?"}

# @(#) subtask fixpx48            Author: R.A. Jansen -- January 17 1996
# @(#) 
# @(#) subtask called from 'ccd48proc' to execute the interpolation over
# @(#) bad lines and columns.
# @(#)
# @(#) Mar  6 2002 -- Headers are now updated  using  'artdata.mkheader'
# @(#)   rather than 'ccdhedit'.  Keywords are now 'FIXPIX' (switch with
# @(#)   value 'PERFORM','OMIT', or 'COMPLETE'), 'FXPXFILE' (a file name
# @(#)   of file containing description of bad pixels and columns),  and
# @(#)   'FXPXDATE' (FITS Y2K compliant date + time string). RAJ.
# @(#) Aug 13 2003 -- Added 'hdrcom' to support header keyword controled
# @(#)   data reduction (a la 'calstis') and added 'verbose' switch. RAJ
# @(#) Jul  6 2004 -- In the case of header directed processing, keyword
# @(#)   a stray 'fixpix02' remained in the header. Fixed this. RAJ.

begin
	string	cimg, cfil, cval

	# Initialize variables...
	cval  = hdrcom
	cimg  = image
	cfil  = fixfile

	# Clean-up any relevant debris...
	if ( access("fixpx.head") ) delete ("fixpx.head", yes, verify-)

	# Check whether image was processed already...
	if ( cval == "COMPLETE" ) {
	   if (verbose) print ("    FIXPIX:   image was processed already; no action taken")
	   print ("  FIXPIX  = \"COMPLETE\"; no action taken", >> logfile)
	} else if ( cval == "OMIT" ) {
	   if (verbose) print ("    FIXPIX  = \"OMIT\"; no action taken")
	   print ("  FIXPIX  = \"OMIT\"; no action taken", >> logfile)

	   # Add header keyword FIXPIX...
	   gdate(utc=yes)
	   hedit (cimg, "fixpix", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "cc48date", "", add-, del+, ver-, show-, upd+)
	   imheader (cimg, longheader+, > "fixpx.head")
	   printf("FIXPIX  = %-21s / Bad pixel interpolation flag\n", "'OMIT'", >> "fixpx.head")
	   print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "fixpx.head")
	   mkheader (cimg, "fixpx.head", append-, verbose-)
	   delete ("fixpx.head", yes, verify-)

	} else {
	   if (verbose) printf("    FIXPIX ...   ")
	   # Interpolate over bad pixels...
	   if ( release < "2.12" ) {
	      fixpix (cimg, cfil, verbose-)
	   } else {
	      fixpix (cimg, cfil, linterp="INDEF", cinterp="INDEF",
			verbose-, pixels-)
	   }
	   if (verbose) printf("done\n")

	   # Add keywords FIXPIX, FIXFILE and log info to image header...
	   gdate(utc=yes)
	   hedit (cimg, "fixpix*", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "fxpxfile", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "fxpxdate", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "cc48date", "", add-, del+, ver-, show-, upd+)
	   imheader (cimg, longheader+, > "fixpx.head")
	   printf("FIXPIX  = %-21s / Bad pixel interpolation flag\n", "'COMPLETE'", >> "fixpx.head")
	   printf("FXPXFILE= %-21s / Bad pixel file\n", "'"//cfil//"'", >> "fixpx.head")
	   print ("FXPXDATE= '"//gdate.fdate//"' / Interpolation over bad pixels done", >> "fixpx.head")
	   print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "fixpx.head")
	   mkheader (cimg, "fixpx.head", append-, verbose-)
	   delete ("fixpx.head", yes, verify-)

	   # Log action to text log file...
	   print ("  FIXPIX:   "//gdate.fdate//" Interpolation over bad pixels in \"fixfile$\" done", >> logfile)

	}

end
