procedure getid48 (image, ccdident, isdir, ncol, nrow, logfile, verbose)

string	image	=""		{prompt="Image name"}
string	ccdident=""		{prompt="CCD detector choice"}
string	isdir	=""		{prompt="Instrument specific directory"}
int	ncol	=INDEF		{prompt="Number of CCD columns (vertical)"}
int	nrow	=INDEF		{prompt="Number of CCD lines (horizontal)"}
string	logfile	=""		{prompt="Text logfile"}
bool	verbose	=yes		{prompt="Verbose messages?"}
string	idccd	=""		{prompt="CCD detector header string"}
string	fixfile	=""		{prompt="File describing bad lines and columns"}
string	shtfile	=""		{prompt="File with shutter-effect coefficients"}
real	rdnoise	=INDEF		{prompt="CCD read out noise (e-)"}
real	gain	=INDEF		{prompt="CCD gain (e-/ADU)"}
real	secpix	=INDEF		{prompt="CCD pixel scale (arcsec/pix)"}
bool	transp	=no		{prompt="Is CCD back-side illuminated?"}
bool	debug	=no		{prompt="Extra verbose (debugging only)?"}
bool	ok	=yes		{prompt="No errors encountered?"}

# @(#) subtask getid48            Author: R.A. Jansen -- January 17 1996
# @(#) 
# @(#) subtask called from 'ccd48proc' to check the detector identifica-
# @(#) tion and see whether a specification (badpixel file) is available.
# @(#)
# @(#) Aug 11 2003 -- Significant rewrite of this task; by requiring the
# @(#)                detector ID in the image headers to be unique,  if
# @(#)                necessary by editting  the original headers,  this
# @(#)                task was made much more efficient.  All escapes to
# @(#)                unix and temporary files were removed and replaced
# @(#)                with native IRAF alternatives (match, scan, etc..)
# @(#) Aug 13 2003 -- Added 'transp' keyword and added boolean column to
# @(#)                the CCD detector files where back-side illuminated
# @(#)                CCDs list 'yes' and front-side ones 'no ' to indi-
# @(#)                cate that the coordinates are transposed  wrt. the
# @(#)                celestial axes. RAJ.
# @(#) Sep 25 2003 -- Added 'shtfile' keyword  to support shutter-effect
# @(#)                correction. RAJ.
# @(#) Apr 10 2004 -- Improved help on running 'ccd48proc' for detectors
# @(#)                that are not in the data base of known ones. RAJ.
# @(#) May 11 2004 -- Minor format change in logfile/message output. RAJ
# @(#) Jul  2 2004 -- Allow gain<1.0 (lower limit is now 0.5). RAJ.

begin
	string	cimg, qid, id, cid, cstr
	struct	cline
	real	ron, eadu, pixsz
	int	nc, nr, nc1, nr1, llen
	bool	bok, bil

#	Initialize variables...
	cimg  = image
	qid   = ccdident
	nc1   = ncol
	nr1   = nrow
	id    = ""
	bok   = ok
	bil   = no

#	(Re)intialize task parameters...
	ok    = bok

#	Return when status on entry was not ok...
	if (!bok) { return }

	if (verbose) printf("\rChecking detector specifications ... ")

#	Get detector ID string from the image header...
	hselect (cimg, "detector", "yes") | scan (cline)
	cid = cline

#	Now, try and search for the queried ccd identification...
	cline = ""
	match (cid, isdir//"ccd.detector", stop-, print-, meta-) | scan (cline)
	cstr = cline
	if ( cstr == "" ) {
	   if (debug) {
	      print ("cstr=\"\"! -- id=\""//cid//"\", ncol=",nc1,", nrow=",nr1, >> logfile)
	      print ("\ncstr=\"\"! -- id=\""//cid//"\", ncol=",nc1,", nrow=",nr1)
	   }
	   print ("INTERUPT: CCD id=\""//cid//"\"? Unknown detector.", >> logfile)
	   print ("\nINTERUPT: CCD id=\""//cid//"\"? Unknown detector.")
	   ok = no
	   return
	} else if ( strlen(cstr) < 58 ) {
	   if (debug) {
	      print ("cstr=\""//cstr//"\"", >> logfile)
	      print ("\ncstr=\""//cstr//"\"")
	   }
	   print ("INTERUPT: CCD id database string truncated (nchar<58)?", >> logfile)
	   print ("\nINTERUPT: CCD id database string truncated (nchar<58)?")
	   ok = no
	   return
	} else {
	   id    = substr(cstr,1,8)
	   nc    = int(substr(cstr,22,26))
	   nr    = int(substr(cstr,28,32))
	   ron   = real(substr(cstr,38,41))
	   eadu  = real(substr(cstr,47,51))
	   pixsz = real(substr(cstr,56,61))
	   bil	 = ( substr(cstr,63,65) == 'yes' )
	}
	if ( nc < 256 || nc > 32768 || nr < 256 || nc > 32768 ) {
	   printf("INTERUPT: image size [%d,%d] outside valid range [256,32768]!\n", nc, nr, >> logfile)
	   printf("\nINTERUPT: image size [%d,%d] outside valid range [256,32768]!\n", nc, nr)
	   ok = no
	   return
	}
	if ( ron < 1.0 || ron > 16. || eadu < 0.5 || eadu > 16. ) {
	   printf("INTERUPT: detector parameters rdnoise=%.2f and gain=%.2f outside valid range [0.5,16.]!\n", ron, eadu, >> logfile)
	   printf("\nINTERUPT: detector parameters rdnoise=%.2f and gain=%.2f outside valid range [0.5,16.]!\n", ron, eadu)
	   ok = no
	   return
	}
	if ( pixsz < 0.1 || pixsz > 1.0 ) {
	   # this package is taylored for reducing ground-based optical data...
	   printf("INTERUPT: ccd pixel scale=%.3f outside valid range [0.1,1.0]!\n", pixsz, >> logfile)
	   printf("\nINTERUPT: ccd pixel scale=%.3f outside valid range [0.1,1.0]!\n", pixsz)
	   ok = no
	   return
	}

#	Presumably we now have a match and managed to parse the database
#	detector specification string correctly. Now find the apropriate
#	bad pixel definition file...
	llen  = strlen(id)
	while ( substr(id,llen,llen) == " " ) { llen = llen - 1 }
	print (substr(id,1,llen) | scan (id)

	if ( qid == "auto" ) {
	   # FLWO 48inch AndyCam CCD camera (Mar 1994--Mar 1997)
	   if ( id == "thin" ) {
	      fixfile = isdir//"badpix.thin"
	      shtfile = isdir//"shutter.thin"
	   } else if ( id == "thick1" ) {
	      fixfile = isdir//"badpix.thick1"
	      shtfile = isdir//"shutter.thick1"
	   } else if ( id == "thick2" ) {
	      fixfile = isdir//"badpix.thick2"
	      shtfile = isdir//"shutter.thick2"
	   } else if ( id == "thicku" ) {
	      fixfile = isdir//"badpix.thicku"
	      shtfile = isdir//"shutter.thicku"
	   } else if ( id == "thin2" ) {
	   # FLWO 60inch FAST spectrograph (Sep 1994--Mar 1997)
	      fixfile = isdir//"badpix.thin2"
	   } else if ( id == "thin4" ) {
	      fixfile = isdir//"badpix.thin4"
	   } else if ( id == "thin4a" ) {
	      fixfile = isdir//"badpix.thin4a"
	   } else if ( id == "thin8" ) {
	      fixfile = isdir//"badpix.thin8"
	   } else if ( id == "thin8a" ) {
	      fixfile = isdir//"badpix.thin8a"
	   } else if ( id == "ccd26" ) {
	   # ESO 3.6m EFOSC1 spectrograph (1995)
	      fixfile = isdir//"badpix.ccd26"
	   } else if ( id == "ccd26x1" ) {
	   # MGIO VATT 1.8m CCD camera (Nov 2001--present)
	      fixfile = isdir//"badpix.ccd26x1"
	      shtfile = isdir//"shutter.ccd26x1"
	   } else if ( id == "ccd26x2" ) {
	      fixfile = isdir//"badpix.ccd26x2"
	      shtfile = isdir//"shutter.ccd26x2"
	   } else if ( id == "ccd26xx" ) {
	      fixfile = isdir//"badpix.ccd26xx"
	      shtfile = isdir//"shutter.ccd26xx"
	   } else {
	      if (debug) {
		 print ("\"auto\"; nrow(1)=",nr1,"  ncol=",nc,"  nrow=",nr,"  pixsz=",pixsz,"  id=\""//id//"\"", >> logfile)
		 print ("\n\"auto\"; nrow(1)=",nr1,"  ncol=",nc,"  nrow=",nr,"  pixsz=",pixsz,"  id=\""//id//"\"")
	      }
	      print ("INTERUPT: CCD id=\""//cid//"\"? Unknown detector.", >> logfile)
	      print ("\nINTERUPT: CCD id=\""//cid//"\"? Unknown detector.")
	      ok = no
	      return
	   }
	} else if ( id != qid ) {
	   print ("INTERUPT: CCD id=\""//cid//"\" ("//id//")", >> logfile)
	   print (" ", >> logfile)
	   print ("  This detector is not in the database of known detectors and, hence,", >> logfile)
	   print ("  the \"auto\" option failed.  You may explicitly enter the detector", >> logfile)
	   print ("  id  in parameter 'ccdident' and the correct instrument specific", >> logfile)
	   print ("  directory in parameter 'isdir'.  Be sure to create the 3 instrument", >> logfile)
	   print ("  specific files \"ccd.detector\", \"ccd.subsets\" and \"badpix.<CCD id>\"", >> logfile)
           print ("  in that directory before running 'ccd48proc'.  A current example", >> logfile)
	   print ("  of the exact format to use may be found in \"ccd48$lib/VATT/\".", >> logfile)
	   print ("  You also need to create a shutter-effect coefficient file named", >> logfile)
	   print ("  \"shutter.<CCD id>\" in 'isdir', which should contain 8 lines of", >> logfile)
	   print ("  floating point numbers (e.g., 0.00000) preceded by an arbitrary", >> logfile)
	   print ("  number of comment lines (having a '#' as their first character).", >> logfile)
	   print ("  Note that this file is required even if no shutter correction is", >> logfile)
	   print ("  performed.", >> logfile)
	   print ("  Contact the author <Rolf.Jansen@asu.edu> if you wish this detector", >> logfile)
	   print ("  to be permanently included in the database of known detectors.", >> logfile)

	   print ("\nINTERUPT: CCD id=\""//cid//"\" ("//id//")")
	   print (" ")
	   print ("  This detector is not in the database of known detectors and, hence,")
	   print ("  the \"auto\" option failed. This need not be a problem. Consult the")
	   print ("  'logfile' for solutions.\n")
	   ok = no
	   return

	} else {
	   # the detector ID was given explicitly...
	   fixfile = isdir//"badpix."//id
	   shtfile = isdir//"shutter."//id
	}

#	Check that the associated bad pixel file is present...
	chkimg (fixfile, "access", imtype="file", verbose-)
	if (!chkimg.ok) {
	   print ("## CCD id =\""//cid//"\" ("//id//")", >> logfile)
	   print ("ERROR: bad pixel file (fixfile) \""//fixfile//"\" not found!", >> logfile)
	   printf("OK                                                    \r")
	   print ("  CCD id =\""//cid//"\" ("//id//")")
	   printf("\rChecking bad pixel file ... ")
	   printf("ERROR                                                 \r")
           print ("  Bad pixel file (fixfile) \""//fixfile//"\" not found!")
	   ok = no
	   return
	} else {
	   print ("## CCD id =\""//cid//"\" ("//id//")", >> logfile)
	   print ("## fixfile=\""//fixfile//"\"", >> logfile)
	   if (verbose) {
	      printf("OK                                                    \r")
	      print ("  CCD id =\""//cid//"\" ("//id//")")
	      printf("\rChecking bad pixel file ... ")
	      printf("         OK                                           \r")
	      print ("  fixfile=\""//fixfile//"\"")
	   }
	}

#	Check that the associated shutter-effect coefficient file is present...
	if ( id != "thin2" && id != "thin4" && id != "thin4a" && \
		id != "thin8" && id != "thin8a" && id != "ccd26" ) {
	   if (verbose) printf("\rChecking shutter coefficients ... ")
	   chkimg (shtfile, "access", imtype="file", verbose-)
	   if (!chkimg.ok) {
	      print ("\nERROR: shutter-effect coefficient file (shtfile) \""//shtfile//"\" not found!", >> logfile)
	      if (!verbose) printf("\rChecking shutter coefficients ... ")
	      printf("ERROR                                                 \r")
	      print ("  Shutter-effect coefficient file (shtfile) \""//shtfile//"\" not found!")
	      ok = no
	      return
	   } else {
	      printf("## shtfile=\""//shtfile//"\"\n\n", >> logfile)
	      if (verbose) {
		 printf("   OK                                                 \r")
	         print ("  shtfile=\""//shtfile//"\"")
	      }
	   }
	} else {
	   print ("\n", >> logfile)
	}

#	Check that number of rows and columns matches the detector specs...
	if (verbose) printf("\rChecking actual image dimensions agains specification ... ")
	#cstr = "" ; hselect (cimg, "overscan", "yes") | scan (cstr)
	#if ( nc != nc1 && ! ( cstr == "COMPLETE" && nc < nc1 ) ) {
	if ( nc != nc1 ) {
	   printf("ERROR: ncol=%d does not match specification (=%d)!\n", nc1, nc, >> logfile)
	   if (!verbose) printf("\rChecking actual image dimensions agains specification ... ")
	   printf("ERROR                                                 \r")
	   printf("  ncol=%d does not match specification (=%d)!\n", nc1, nc)
	   ok = no
	   return
	}
	if ( nr != nr1 ) {
	   printf("ERROR: nrow=%d does not match specification (=%d)!\n", nr1, nr, >> logfile)
	   if (!verbose) printf("\rChecking actual image dimensions agains specification ... ")
	   printf("ERROR                                                 \r")
	   printf("  nrow=%d does not match specification (=%d)!\n", nr1, nr)
	   ok = no
	   return
	}
	printf("OK                                                    \r")

#	Update remaining task parameters...
	idccd	= id
	rdnoise	= ron
	gain	= eadu
	secpix	= pixsz
	transp	= bil

end
