procedure gimst48 (inlist, isfile, imgtyp, hdrproc, nimg, ncol, nrow, maxval,
		subset, expkey, logfile, getstat)

string	inlist	=""		{prompt="Input list of image names"}
string	isfile	=""		{prompt="Instrument subset file"}
string	imgtyp	=""		{prompt="Image type"}
bool	hdrproc	=no		{prompt="Processing directed by header keywords?"}
int	nimg	=INDEF		{prompt="Number of images in input list"}
int	ncol	=INDEF		{prompt="Number of columns (set by CCD id)"}
int	nrow	=INDEF		{prompt="Number of lines (set by CCD id)"}
real	maxval	=INDEF		{prompt="Highest valid pixel value"}
string	subset	=""		{prompt="Subset identifier header keyword"}
string	expkey	="exptime"	{prompt="Exposure time header keyword"}
string	logfile =""		{prompt="Text log file"}
bool	getstat	=no		{prompt="Compute image statistics?"}
string	statsec	=""		{prompt="Section for computing statistics"}
string	biassec	=""		{prompt="CCD overscan image section"}
string	statlog	=""		{prompt="Statistics log file"}
bool	verbose	=no		{prompt="Verbose?"}
string	drkkey	="darktime"	{prompt="Dark time header keyword"}
bool	ok	=yes		{prompt="No errors encountered?"}

# @(#) subtask gimst48            Author: R.A. Jansen -- January 22 1996
# @(#) 
# @(#) subtask called from 'ccd48proc' to check whether the subset given
# @(#) by header keyword 'subset' is known (ie. has an entry in the data
# @(#) base) and, if so, to create one image list per subset encountered.
# @(#) Optionally,  the image statistics in data- and bias- sections are
# @(#) computed and logged to 'statlog' for future reference.
# @(#)
# @(#) Aug 12 2003 -- Rewrote and merged this task with 'gsubs48', which
# @(#)                is now obsolete, and moved several checks from the
# @(#)                main procedure to this task to streamline the ini-
# @(#)                tial checks of the images and image headers. Added
# @(#)                parameters 'nimg','maxval','subset','getstat', and
# @(#)                'verbose'. RAJ.
# @(#) Aug 16 2003 -- Added 'expkey' and 'darkkey' parameters. RAJ.
# @(#) Apr 19 2004 -- Absent 'filter' header keyword in the case of bias
# @(#)                or dark frames  did not trigger an error,  causing
# @(#)                ccd48proc to bail.  Added test and changed default
# @(#)                to add FILTER='D' when absent. RAJ.
# @(#) Jul  6 2004 -- Added 'hdrproc' parameter in support of processing
# @(#)                directed by header parameters. RAJ.

begin
	string	clis, cfil, type, ssec, bsec, ckey, csub, cexp, cdrk
	string	cnam, ctyp, ctmp
	int	nl, nc, nr, ns, i, j, ifrc
	real	texp, tdrk, ravg, rmax
	bool	bhdr, bstat, match, bok, sok, aok

#	Initialize variables...
	clis  = inlist
	cfil  = isfile
	type  = imgtyp
	bhdr  = hdrproc
	nl    = nimg
	nc    = ncol
	nr    = nrow
	rmax  = maxval
	ckey  = subset
	cexp  = expkey
	cdrk  = drkkey
	bstat = getstat
	ssec  = statsec
	bsec  = biassec
	aok   = yes

#	(Re)initialize task parameter...
	ok   = yes

#	Open logfile and start logging...
 	sysinfo() ; gdate(utc=no)
	if (bstat) {
	   print ("\n## CCD48PROC (GIMST48)  "//sysinfo.sysmsg//"  "//gdate.tdate, >> statlog)
	   print ("## Input=\""//clis//"\",  imgtyp=\""//type//"\"\n", >> statlog)
	   printf("#%21s%44t%7s %7s %7s %6s %6s\n", "IMAGE", "NPIX", "MEAN",
		"STDDEV", "MIN", "MAX", >> statlog)
	   if (verbose) print ("Checking image subsets and statistics ... ")
	} else if (verbose) {
	   print ("Checking image subsets ... ")
	}

#	Count number of subsets in instrument specific file...
	iwc (cfil)
	ns = iwc.nlines

#	Loop over all images in the input list...
	for ( i = 1 ; i <= nl ; i = i + 1 ) {
	   bok = yes
	   sok = yes

	   rdlist (clis, i)
	   cnam = rdlist.cline
	   chkimg (cnam, "access", imtype="", verbose-)
	   cnam = chkimg.impath//chkimg.imroot
	   ctyp = chkimg.imtype

	   # test that 'imagetyp' and exposure/dark time keywords are
	   # present so that we may skip this test in the main routine;
	   # also make sure that 'imagetyp' is given in upper case and
	   # matches 'type'...
	   ctmp = ""
	   hselect (cnam//ctyp, "imagetyp", "yes") | scan (ctmp)

	   if ( ctmp == "" ) {
	      print (cnam//ctyp//": header keyword \"IMAGETYP\" is missing!", >> logfile)
	      print (cnam//ctyp//": header keyword \"IMAGETYP\" is missing!")
	      aok = no
	   }
	   if ( ctmp=="zero" || ctmp=="Zero" || ctmp=="bias" || ctmp=="Bias" ) {
	      hedit (cnam//ctyp, "imagetyp", "ZERO", add-, delete-,
			verify-, show-, update+)
	      ctmp = "ZERO"
	   }
	   if ( ctmp=="dark" || ctmp=="Dark" ) {
	      hedit (cnam//ctyp, "imagetyp", "DARK", add-, delete-,
			verify-, show-, update+)
	      ctmp = "DARK"
	   }
	   if ( ctmp=="flat" || ctmp=="Flat" || ctmp=="FLAT" ) {
	      if ( type == "SKY" ) {
		 hedit (cnam//ctyp, "imagetyp", "SKY", add-, delete-,
			verify-, show-, update+)
		 ctmp = "SKY"
	      } else {
		 hedit (cnam//ctyp, "imagetyp", "FLAT", add-, delete-,
			verify-, show-, update+)
		 ctmp = "FLAT"
	      }
	   }
	   if ( ctmp=="sky" || ctmp=="Sky" ) {
	      hedit (cnam//ctyp, "imagetyp", "SKY", add-, delete-,
			verify-, show-, update+)
	      ctmp = "SKY"
	   }
	   if ( ctmp=="object" || ctmp=="Object" || ctmp=="OBJECT" ) {
	      if ( type == "SKY" ) {
		 hedit (cnam//ctyp, "imagetyp", "SKY", add-, delete-,
			verify-, show-, update+)
		 ctmp = "SKY"
	      } else {
		 hedit (cnam//ctyp, "imagetyp", "OBJECT", add-, delete-,
			verify-, show-, update+)
		 ctmp = "OBJECT"
	      }
	   }
	   if ( type == "ZERO" && ctmp != "ZERO" ) bok = no
	   if ( type == "DARK" && ctmp != "DARK" ) bok = no
	   if ( type == "FLAT" && ctmp != "FLAT" ) bok = no
	   if ( type == "SKY"  && ctmp != "SKY"  ) bok = no
	   if ( type == "OBJECT" && ctmp != "OBJECT" ) bok = no
	   if ( bok == no ) {
	      print (cnam//ctyp//": Unsupported imagetyp=\""//ctmp//"\" for type=\""//type//"\"!", >> logfile)
	      print (cnam//ctyp//": Unsupported imagetyp=\""//ctmp//"\" for type=\""//type//"\"!")
	      aok = no
	   }

	   texp = -1.
	   tdrk = -1.
	   hselect (cnam//ctyp, cexp, "yes") | scan (texp)
	   if ( texp < 0 ) {
	      print (cnam//ctyp//": header keyword \"EXPTIME\" is missing!", >> logfile)
	      print (cnam//ctyp//": header keyword \"EXPTIME\" is missing!")
	      aok = no
	   }
	   hselect (cnam//ctyp, cdrk, "yes") | scan (tdrk)
	   if ( tdrk < 0 ) {
	      drkkey = cexp
	      tdrk = texp
	   } else {
	      drkkey = cdrk
	   }

	   # try and find a match for the subset in the subset table for
	   # flat fields and object exposures, and set the subset to a
	   # common arbitrary subset ("D") if it is a frame depending only
	   # on the electronic characteristics of the detector (biases,
	   # darks, and analog-to-digital/linearity exposures).
	   csub = ""
	   hselect (cnam//ctyp, ckey, "yes") | scan (csub)
	   if ( type == "FLAT" || type == "SKY" || type == "OBJECT" ) {
	      if ( csub == "" ) {
		 print (cnam//ctyp//": header keyword \""//ckey//"\" is missing!", >> logfile)
		 print (cnam//ctyp//": header keyword \""//ckey//"\" is missing!")
		 aok = no
	      }
	      match = no
	      for ( j = 1 ; j <= ns ; j = j + 1 ) {
		 rdlist (cfil, j)
		 if ( csub == rdlist.cline ) { match = yes ; break }
	      }
	      if (match) {
	         if ( type == "OBJECT" ) {
		    if ( i == 1 && access("data_"//csub//".lis") ) {
			delete ("data_"//csub//".lis", yes, verify-)
		    }
		    print (cnam//csub//ctyp, >> "data_"//csub//".lis")
	         } else {
		    if ( i == 1 && access(type//"_"//csub//".lis") ) {
			delete (type//"_"//csub//".lis", yes, verify-)
		    }
		    print (cnam//csub//ctyp, >> type//"_"//csub//".lis")
	         }
	      } else {
	         print (cnam//ctyp//"["//csub//"]: SUBSET not in subset table, frame not processed!", >> logfile)
	         print (cnam//ctyp//"["//csub//"]: SUBSET not in subset table, frame not processed!")
	         aok = no
	      }
	   } else if ( type == "ZERO" || type == "DARK" ) {
	      if ( csub == "" ) {
		 csub = "D"
	         hedit (cnam//ctyp, ckey, csub, add+, del-, ver-, show-, update+)
		 print (cnam//ctyp//": header keyword \""//ckey//"\" is missing; assuming \""//ckey//"\"='"//csub//"'", >> logfile)
		 print (cnam//ctyp//": header keyword \""//ckey//"\" is missing; assuming \""//ckey//"\"='"//csub//"'")
	      } else {
	         csub = "D"
	         hedit (cnam//ctyp, ckey, csub, add-, del-, ver-, show-, update+)
	      }
	      if ( i == 1 && access(type//".lis") ) {
		 delete (type//".lis", yes, verify-)
	      }
	      print (cnam//ctyp, >> type//".lis")
	   } else {
	      if ( csub == "" ) csub = "D"
	      print (cnam//ctyp//"["//csub//"]["//type//"]: Unsupported image type \""//type//"\"!", >> logfile)
	      print (cnam//ctyp//"["//csub//"]["//type//"]: Unsupported image type \""//type//"\"!")
	      aok = no
	   }

	   # If the processing will be directed by header keywords, check
	   # whether a BIASSEC and/or TRIMSEC keyword is present ...
	   if (bhdr) {
	      ctmp = "" ; hselect (cnam//ctyp, "biassec", "yes") | scan (ctmp)
	      if ( ctmp != "" ) { bsec = ctmp }
	      ctmp = "" ; hselect (cnam//ctyp, "trimsec", "yes") | scan (ctmp)
	      if ( ctmp != "" ) { ssec = ctmp }
	   }

	   # Update task parameters...
	   statsec = ssec
	   biassec = bsec

	   # Get an estimate of the mean image pixel value in image- and
	   # overscan regions and (optionally) log the image statistics
	   # for both regions to 'statlog'...
	   imgstat (cnam//ctyp//ssec, lower=INDEF, upper=INDEF, verbose-)
	   ravg = imgstat.mean
	   if (bstat) {
	      ctmp = cnam//ssec//"["//csub//"]["//type//"]"
	      printf("%-44s%7d %7.1f %6.2f %6.0f %6.0f\n", ctmp, imgstat.npix,
		ravg, imgstat.stddev, imgstat.min, imgstat.max, >> statlog)
	   }
	   imgstat (cnam//ctyp//bsec, lower=INDEF, upper=INDEF, verbose-)
	   if (bstat) {
	      ctmp = cnam//bsec//"["//csub//"]["//type//"]"
	      printf("%-44s%7d %7.1f %6.2f %6.0f %6.0f\n", ctmp, imgstat.npix,
		imgstat.mean, imgstat.stddev, imgstat.min, imgstat.max, >> statlog)
	   }

	   # Test whether 'type' and 'texp'/'tdrk' and/or the mean count
	   # level are consistent with one another...
	   if ( type == "ZERO" && texp > 0. ) {
	      print (cnam//ctyp//"["//csub//"]["//type//"]: EXPTIME > 0.!", >> logfile)
	      print (cnam//ctyp//"["//csub//"]["//type//"]: EXPTIME > 0.!")
	      sok = no
	   }
	   if ( type == "DARK" && tdrk < 600 ) {
	      print (cnam//ctyp//"["//csub//"]["//type//"]: DARKTIME < 600!", >> logfile)
	      print (cnam//ctyp//"["//csub//"]["//type//"]: DARKTIME < 600!")
	      sok = no
	   }
	   if ( type != "ZERO" && type != "DARK" && texp < 0.1 ) {
	      print (cnam//ctyp//"["//csub//"]["//type//"]: EXPTIME < 0.1!", >> logfile)
	      print (cnam//ctyp//"["//csub//"]["//type//"]: EXPTIME < 0.1!")
	      sok = no
	   }
	   if ( ( type == "FLAT" || type == "SKY" ) && (ravg-imgstat.mean) < 1000. ) {
	      printf(cnam//ctyp//"["//csub//"]["//type//"]: MEAN=%8f < 1000.!\n", (ravg-imgstat.mean), >> logfile)
	      printf(cnam//ctyp//"["//csub//"]["//type//"]: MEAN=%8f < 1000.!\n", (ravg-imgstat.mean))
	      sok = no
	   }
	   if ( ( type == "FLAT" || type == "SKY" ) && ravg > rmax ) {
	      printf(cnam//ctyp//"["//csub//"]["//type//"]: saturated or non-linear (MEAN=%8f > %8f)!\n", ravg, rmax, >> logfile)
	      printf(cnam//ctyp//"["//csub//"]["//type//"]: saturated or non-linear (MEAN=%8f > %8f)!\n", ravg, rmax)
	      sok = no
	   }

	   if ( !bok || !sok ) aok = no

	   ifrc = nint(100.*i/nl)
	   if ( (ifrc % 2 ) == 0 ) printf("\r %3d %1s done...\r", ifrc, "%")
	}
	printf("\r                                                      \r")

#	Check whether there were problems detected that would require
#	action by the user...
	if ( !aok ) {
	   print ("Fix problems noted, or remove affected images from the input list.", >> logfile)
	   print ("Fix problems noted, or remove affected images from the input list.")
	   ok = no
	}

#	Closing message...
	if ( bstat && verbose ) {
	   printf("OK                                                    \r")
	   print ("  Image statistics logged to file \""//statlog//"\".")
	} else {
	  if (verbose) printf("OK                                                    \r")
	}

end
