procedure ilcor48 (hdrcom, image, illum, subset, logfile, flatlog,
		verbose, first)

string	hdrcom	=""		{prompt="Header controled command"}
string	image	=""		{prompt="Image name"}
string	illum	=""		{prompt="Illumination frame (root)name"}
string	subset	=""		{prompt="Image subset"}
string	logfile	=""		{prompt="Text logfile ccd48proc"}
string	flatlog	=""		{prompt="Text logfile flatfielding"}
bool	verbose	=yes		{prompt="Verbose messages?"}
bool	first	=no		{prompt="First image in list?"}

# @(#) subtask ilcor48            Author: R.A. Jansen -- January 17 1996
# @(#) 
# @(#) subtask called from 'ccd48proc' to correct for illumination diff-
# @(#) erences over the CCD, using an illumination calibration frame.
# @(#)
# @(#) Nov  4 2002 -- Headers are now updated  using  'artdata.mkheader'
# @(#)   rather than 'ccdhedit'. Keywords are now 'ILLUMCOR' (with value
# @(#)   'PERFORM','OMIT', or 'COMPLETE'), 'ILCRIMAG' with image name as
# @(#)   its value, and 'ILCRDATE' (FITS Y2K compliant date+time string).
# @(#) Aug 14 2003 -- Added 'hdrcom' to support header keyword controled
# @(#)   data reduction (a la 'calstis') and added 'verbose' switch. RAJ
# @(#) Apr 21 2004 -- Fixed bug of ILCRIMAG header record >80 chars, and
# @(#)   bug of double subset identifier.  Added check that illumination
# @(#)   flat is normalized; if not, (re)normalize. RAJ

begin
	string	cval, cimg, iimg, csub, cstr
	real	rmean

	# Initialize variables...
	cval  = hdrcom
	cimg  = image
	iimg  = illum
	csub  = subset
	rmean = INDEF
	if ( flatlog == "" ) flatlog = "flatproc.log"

	# Clean-up any relevant debris...
	if ( access("ilcor.head") ) delete("ilcor.head", yes, verify-)

	# If first image in list, open flatlog and start logging...
	if (first) {
	   sysinfo() ; gdate(utc=yes)
	   if ( access(flatlog) ) {
	      print ("\n## CCD48PROC (ILCOR48)  "//sysinfo.sysmsg//"  "//gdate.tdate//"\n", >> flatlog)
	   } else {
	      print ("## CCD48PROC (ILCOR48)  "//sysinfo.sysmsg//"  "//gdate.tdate//"\n", >> flatlog)
	   }
	}

	# Check whether image was processed already...
	if ( cval == "COMPLETE" ) {
	   if (verbose) print ("    ILLUMCOR: image was processed already; no action taken")
	   print ("  ILLUMCOR= \"COMPLETE\"; no action taken", >> logfile)
	   print (cimg//": ILLUMCOR= \"COMPLETE\"; no action taken", >> flatlog)

	} else if ( cval == "OMIT" ) {
	   if (verbose) print ("    ILLUMCOR= \"OMIT\"; no action taken")
	   print ("  ILLUMCOR= \"OMIT\"; no action taken", >> logfile)
	   print (cimg//": ILLUMCOR= \"OMIT\"; no action taken", >> flatlog)

	   # Add header keyword ILLUMCOR...
	   gdate(utc=yes)
	   hedit (cimg, "illumcor", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "cc48date", "", add-, del+, ver-, show-, upd+)
	   imheader (cimg, longheader+, > "ilcor.head")
	   printf("ILLUMCOR= %-21s / Illumination correction flag\n", "'OMIT'", >> "ilcor.head")
	   print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "ilcor.head")
	   mkheader (cimg, "ilcor.head", append-, verbose-)
	   delete ("ilcor.head", yes, verify-)

	} else {
	   if ( verbose && !first ) printf("    ILLUMCOR ...")

	   # Test whether appropriate illumination frame is present...
	   chkimg (iimg//csub, "access", imtype="", verbose-)
	   iimg = chkimg.impath//chkimg.imroot
	   if (!chkimg.ok) {
	      print ("    ILLUMCOR... illum. image \""//iimg//chkimg.imtype//"\" not found; no action taken")
	      print ("  ILLUMCOR: illumination image \""//iimg//chkimg.imtype//"\" not found; no action taken", >> logfile)
	      print (cimg//": illumination image \""//iimg//chkimg.imtype//"\" not found; no action taken", >> flatlog)
	      return
	   }

	   # Test whether illumination frame has mean=1; if not, renormalize...
	   if (first) {
	      imstat (iimg, fields="mean", lower=INDEF, upper=INDEF,
			nclip=3., lsigma=4., usigma=4., binwidth=0.1,
			format-, cache-) | scan(rmean)

	      if ( rmean == INDEF || rmean == 0. ) rmean = 1.
	      if ( abs(rmean-1.) > 1e-3 ) {
		 imarith (iimg, "/", rmean, iimg, title="", divzero=0.,
			hparams="", pixtype="real", calctype="real",
			verbose-, noact-)

		 # Add keywords NORMALIZ, NMLZMEAN, and NMLZDATE to image header
		 if ( access("fnorm.head") ) delete ("fnorm.head", yes, verify-)
		 gdate(utc=yes)
		 hedit (iimg, "normaliz", "", add-, del+, ver-, show-, upd+)
		 hedit (iimg, "nmlzmean", "", add-, del+, ver-, show-, upd+)
		 hedit (iimg, "nmlzdate", "", add-, del+, ver-, show-, upd+)
		 hedit (iimg, "cc48date", "", add-, del+, ver-, show-, upd+)
		 imheader (iimg, longheader+, > "fnorm.head")
		 printf("NORMALIZ= %-21s / Illumination flat normalization flag\n", >> "fnorm.head")
		 printf("NMLZMEAN= %20.4f  / Mean level (ADU) prior to normalization\n", rmean, >> "fnorm.head")
		 print ("NMLZDATE= '"//gdate.fdate//"' / Illumination flat normalization done", >> "fnorm.head")
		 print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "fnorm.head")
		 mkheader (iimg, "fnorm.head", append-, verbose-)
		 delete ("fnorm.head", yes, verify-)

		 # Log action to text log files and screen...
		 print ("    ILLUMNORM ...")
		 printf("       Renormalizing \""//iimg//"\" (mean=%.4f-->1.0000) ...\n", rmean)
		 printf("  ILLUMNORM: renormalized \""//iimg//"\" (mean=%.4f-->1.0000)\n", rmean, >> logfile)
		 printf("Renormalized \""//iimg//"\" (mean=%.4f-->1.0000) ...\n", rmean, >> flatlog)
	      }

	      if (verbose) printf("    ILLUMCOR ...")
	   }

	   # Divide image by the appropriate illumination frame...
	   imarith (cimg, "/", iimg, cimg, title="", divzero=0., hparams="",
		pixtype="real", calctype="real", verbose+, noact-, >> flatlog)

	   # Add keywords ILLUMCOR, ILCRIMAG and ILCRDATE to image header...
	   gdate(utc=yes)
	   hedit (cimg, "illumcor", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "ilcrimag", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "ilcrdate", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "cc48date", "", add-, del+, ver-, show-, upd+)
	   imheader (cimg, longheader+, > "ilcor.head")
	   printf("ILLUMCOR= %-21s / Illumination correction flag\n", "'COMPLETE'", >> "ilcor.head")
	   if ( strlen(iimg) <= 21 ) {
	      cstr = "'"//iimg//"'"
	      printf("ILCRIMAG= %-21s / Name of illumination image\n", cstr, >> "ilcor.head")
	   } else {
	      if ( strlen(iimg) <= 39 ) {
		 cstr = "'"//iimg//"'"
	      } else {
		 cstr = "'..."//substr(iimg,(strlen(iimg)-35),strlen(iimg))//"'"
	      }
	      printf("ILCRIMAG= %-"//strlen(cstr)//"s / Name of illumination image\n", cstr, >> "ilcor.head")
	   }
	   print ("ILCRDATE= '"//gdate.fdate//"' / Illumination correction done", >> "ilcor.head")
	   print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "ilcor.head")
	   mkheader (cimg, "ilcor.head", append-, verbose+)
	   delete ("ilcor.head", yes, verify-)

	   if (verbose) print (" done (subs=\""//csub//"\")\n")

	   # Log action to text log files...
	   print ("  ILLUMCOR: "//gdate.fdate//" Illumination correction using image "//cstr, >> logfile)
	   print (cimg//":", >> flatlog)
	   print ("  ILLUMCOR: "//gdate.fdate//" Illumination correction using image "//cstr, >> flatlog)

	}

end
