procedure ptest48 (logfile, ncol, nrow, fixpix, zerocor, oversca, trim,
		darkcor, shutcor, flatcor, illumcor, scatcor)

string	logfile	=""	{prompt="Text logfile (for error messages)"}
int	ncol	=INDEF	{prompt="Number of columns"}
int	nrow	=INDEF	{prompt="Number of rows"}
bool	fixpix	=no	{prompt="Fix bad CCD lines and columns?"}
bool	zerocor	=no	{prompt="Apply bias correction using zero image?"}
bool	oversca	=no	{prompt="Subtract bias level using overscan strip?"}
bool	trim	=no	{prompt="Trim the image?"}
bool	darkcor	=no	{prompt="Apply dark count correction?"}
bool	shutcor	=no	{prompt="Apply shutter correction?"}
bool	flatcor	=no	{prompt="Apply flatfield correction?"}
bool	illumcor=no	{prompt="Apply illumination correction?"}
bool	scatcor	=no	{prompt="Apply scattered light correction?"}
string	isdir	=""	{prompt="Instrument specific directory"}
string	isfil	=""	{prompt="Instrument specific file"}
string	zero	=""	{prompt="Zero level calibration image"}
string	biassec	=""	{prompt="Overscan section"}
string	trimsec	=""	{prompt="Image section"}
string	dark	=""	{prompt="Dark count calibration image"}
string	shut	=""	{prompt="Shutter effect correction image"}
string	flat	=""	{prompt="Flatfield image(s) root name"}
string	illum	=""	{prompt="Illumination image(s) root name"}
string	scat	=""	{prompt="Scattered light image(s) root name"}
int	maxcomb	=INDEF	{prompt="Max. # of images to combine at once"}
bool	ok	=yes	{prompt="Are parameters consistent?"}

# @(#) subtask ptest48            Author: R.A. Jansen -- January 17 1996
# @(#) 
# @(#) subtask called from  'ccd48proc'  to check the consistency of the 
# @(#) parameters defining the operations to be executed by 'ccd48proc'.
# @(#)
# @(#) Feb  6 1998 -- Changed 'access' checks by calls to 'chkimg'. RAJ.
# @(#) Jul 13 2000 -- Added support for image format other than OIF. RAJ
# @(#) Aug 11 2003 -- Added 'scatcor', 'scat', 'ncol' and 'nrow' parame-
# @(#)                ters,  and inserted check of validity of bias- and
# @(#)                trim-section strings  (presently no longer part of
# @(#)                the main routine, 'ccd48proc'. RAJ.
# @(#) Aug 30 2003 -- Added 'maxcomb' parameter and test of total memory
# @(#)                (RAM) available on the host machine. RAJ.
# @(#) Apr  9 2004 -- Added 'shutcor' parameter; minor bug fixes. RAJ.
# @(#) Apr 28 2004 -- Minor bug fix in format of !free call. RAJ+AJM.

begin
	string	cisdir, cisfil, czero, cdark, cshut, cflat, cillum, cscat
	string	bsec, tsec
	int	nc, nr, mx, ns, xlo, xhi, ylo, yhi, i, j
	bool	bok

#	Initialize variables...
	nc     = ncol
	nr     = nrow
	cisdir = isdir
	cisfil = isfil
	czero  = zero
	bsec   = biassec
	tsec   = trimsec
	cdark  = dark
	cshut  = shut
	cflat  = flat
	cillum = illum
	cscat  = scat
	mx     = maxcomb
	bok    = ok

#	(Re)initialize task parameter...
	ok   = bok

#	Return when status on entry was not ok...
	if (!bok) { return }

#	Parameter definition tests ...
# 	-- Instrument specific directory:
	chkimg (cisdir, "access", imtype="file", verbose-)
	if (!chkimg.ok) {
	   ok = no
	   print ("ERROR: instrument specific directory \""//chkimg.impath//"\" not found!", >> logfile)
	   error (1, "instrument specific directory \""//chkimg.impath//"\" not found!")
	} else {
	   cisdir = chkimg.impath
	}

#	-- Instrument specific file:
	chkimg (cisfil, "access", imtype="file", verbose-)
	if (!chkimg.ok) {
	   ok = no
	   print ("ERROR: instrument specific file \""//chkimg.impath//chkimg.imroot//"\" not found!", >> logfile)
	   error (1, "instrument specific file \""//chkimg.impath//chkimg.imroot//"\" not found!")
	} else {
	   cisfil = chkimg.impath//chkimg.imroot
	   iwc (chkimg.impath//chkimg.imroot)
	   ns = iwc.nlines
	   if ( ns == 0 ) {
	      print ("ERROR: \""//cisfil//"\" is empty!", >> logfile)
	      print ("ERROR: \""//cisfil//"\" is empty!")
	      ok = no
	      return
	   }
	   if ( ns != iwc.nwords ) {
	      print ("ERROR: \""//cisfil//"\" may have more than one subset per line,", >> logfile)
	      print ("       may have been created with 'ls' aliased to '\ls --color',", >> logfile)
	      print ("       or has leading and/or trailing blank lines!", >> logfile)
	      print ("ERROR: \""//cisfil//"\" may have more than one subset per line,")
	      print ("       may have been created with 'ls' aliased to '\ls --color',")
	      print ("       or has leading and/or trailing blank lines!")
	      ok = no
	      return
	   }
	}

#	-- Zero level calibration image:
	if ( zerocor && czero == "" ) {
	   print ("INTERUPT: zerocor=yes, while zero=\"\"!", >> logfile)
	   print ("INTERUPT: zerocor=yes, while zero=\"\"!")
	   ok = no
	   return
	}
	if ( zerocor ) {
	   chkimg (czero, "access", imtype="", verbose-)
	   if (!chkimg.ok) {
	      ok = no
	      print ("ERROR: zero level calibration image \""//chkimg.impath//chkimg.imroot//chkimg.imtype//"\" not found!", >> logfile)
	      error (1, "zero level calibration image \""//chkimg.impath//chkimg.imroot//chkimg.imtype//"\" not found!")
	   } else {
	      zero = chkimg.impath//chkimg.imroot//chkimg.imtype
	   }
	}

#	-- Overscan section:
	if ( oversca && bsec == "" ) {
	   print ("INTERUPT: oversca=yes, while biassec=\"\"!", >> logfile)
	   print ("INTERUPT: oversca=yes, while biassec=\"\"!")
	   ok = no
	   return
	}
	if ( oversca ) {
	   rpparse (bsec, verbose-)
	   if (!rpparse.ok) {  
	      print ("ERROR: Invalid biassec = \""//bsec//"\"!", >> logfile)
	      print ("ERROR: Invalid biassec = \""//bsec//"\"!")
	      ok = no
	      return
	   }
	   xlo = rpparse.xlo
	   xhi = rpparse.xhi
	   ylo = rpparse.ylo
	   yhi = rpparse.yhi
	   if ( rpparse.xlo == INDEF ) xlo = 1
	   if ( rpparse.xhi == INDEF ) xhi = nc
	   if ( rpparse.ylo == INDEF ) ylo = 1
	   if ( rpparse.yhi == INDEF ) yhi = nr
	   rpbuild (xlo, xhi, ylo, yhi, pln=0)
	   bsec = rpbuild.imrpstr
	   if ( xlo < 1 || xhi > nc || ylo < 1 || yhi > nr ) {
	      print ("ERROR: biassec = \""//bsec//"\": Pixels out of bounds!", >> logfile)
	      print ("ERROR: biassec = \""//bsec//"\": Pixels out of bounds!")
	      ok = no
	      return
	   }
	}

#	-- Trim section:
	if ( trim && trimsec == "" ) {
	   print ("INTERUPT: trim=yes, while trimsec=\"\"!", >> logfile)
	   print ("INTERUPT: trim=yes, while trimsec=\"\"!")
	   ok = no
	   return
	}
	if ( trim ) {
	   rpparse (tsec, verbose-)
	   if (!rpparse.ok) {  
	      print ("ERROR: Invalid trimsec = \""//tsec//"\"!", >> logfile)
	      print ("ERROR: Invalid trimsec = \""//tsec//"\"!")
	      ok = no
	      return
	   }
	   xlo = rpparse.xlo
	   xhi = rpparse.xhi
	   ylo = rpparse.ylo
	   yhi = rpparse.yhi
	   if ( rpparse.xlo == INDEF ) xlo = 1
	   if ( rpparse.xhi == INDEF ) xhi = nc
	   if ( rpparse.ylo == INDEF ) ylo = 1
	   if ( rpparse.yhi == INDEF ) yhi = nr
	   rpbuild (xlo, xhi, ylo, yhi, pln=0)
	   tsec = rpbuild.imrpstr
	   if ( xlo < 1 || xhi > nc || ylo < 1 || yhi > nr ) {
	      print ("ERROR: trimsec = \""//tsec//"\": Pixels out of bounds!", >> logfile)
	      print ("ERROR: trimsec = \""//tsec//"\": Pixels out of bounds!")
	      ok = no
	      return
	   }
	}

#	-- Dark count calibration image:
	if ( darkcor && cdark == "" ) {
	   print ("INTERUPT: darkcor=yes, while dark=\"\"!", >> logfile)
	   print ("INTERUPT: darkcor=yes, while dark=\"\"!")
	   ok = no
	   return
	}
	if ( darkcor ) {
	   chkimg (cdark, "access", imtype="", verbose-)
	   if (!chkimg.ok) {
	      ok = no
	      print ("ERROR: dark count calibration image \""//chkimg.impath//chkimg.imroot//chkimg.imtype//"\" not found!", >> logfile)
	      error (1, "dark count calibration image \""//chkimg.impath//chkimg.imroot//chkimg.imtype//"\" not found!")
	   } else {
	      dark = chkimg.impath//chkimg.imroot//chkimg.imtype
	   }
	}

#	-- Shutter effect correction image:
	if ( shutcor && cshut == "" ) {
	   print ("INTERUPT: shutcor=yes, while shut=\"\"!", >> logfile)
	   print ("INTERUPT: shutcor=yes, while shut=\"\"!")
	   ok = no
	   return
	}
	if ( shutcor ) {
	   chkimg (cshut, "access", imtype="", verbose-)
	   if (!chkimg.ok) {
	      ok = no
	      print ("ERROR: shutter effect correction image \""//chkimg.impath//chkimg.imroot//chkimg.imtype//"\" not found!", >> logfile)
	      error (1, "shutter effect correction image \""//chkimg.impath//chkimg.imroot//chkimg.imtype//"\" not found!")
	   } else {
	      shut = chkimg.impath//chkimg.imroot//chkimg.imtype
	   }
	}

#	-- Flat field calibration image(s).  NOTE: we cannot a priori
#	verify whether a flat field image for a specific filter for which
#	science exposures may exist is present, only whether any flat
#	field images with the proper image root name and extension are
#	present.
	if ( flatcor && cflat == "" ) {
	   print ("INTERUPT: flatcor=yes, while flat=\"\"!", >> logfile)
	   print ("INTERUPT: flatcor=yes, while flat=\"\"!")
	   ok = no
	   return
	}
	if ( flatcor ) {
	   chkimg (cflat, "access", imtype="", verbose-)
	   i = 0 ; j = 0
	   dir (chkimg.impath//chkimg.imroot//"*"//chkimg.imtype, long-,
		ncols=1) | count | scan (i,j)
	   if ( i == 0 || ( i == 1 && j == 3 ) ) {
	      # dir returned "No files found"
	      ok = no
	      print ("ERROR: no flat field images \""//chkimg.impath//chkimg.imroot//"*"//chkimg.imtype//"\" found!", >> logfile)
	      error (1, "no flat field images \""//chkimg.impath//chkimg.imroot//"*"//chkimg.imtype//"\" found!")
	   } else {
	      flat = chkimg.impath//chkimg.imroot
	   }
	}

#	-- Illumination correction image(s). See note for flat fields.
	if ( illumcor && cillum == "" ) {
	   print ("INTERUPT: illumcor=yes, while illum=\"\"!", >> logfile)
	   print ("INTERUPT: illumcor=yes, while illum=\"\"!")
	   ok = no
	   return
	}
	if ( illumcor ) {
	   chkimg (cillum, "access", imtype="", verbose-)
	   i = 0 ; j = 0
	   dir (chkimg.impath//chkimg.imroot//"*"//chkimg.imtype, long-,
		ncols=1) | count | scan (i,j)
	   if ( i == 0 || ( i == 1 && j == 3 ) ) {
	      # dir returned "No files found"
	      ok = no
	      print ("ERROR: no illumination images \""//chkimg.impath//chkimg.imroot//"*"//chkimg.imtype//"\" found!", >> logfile)
	      error (1, "no illumination images \""//chkimg.impath//chkimg.imroot//"*"//chkimg.imtype//"\" found!")
	   } else {
	      illum = chkimg.impath//chkimg.imroot
	   }
	}

#	-- Scattered light correction image(s). See note for flat fields.
	if ( scatcor && cscat == "" ) {
	   print ("INTERUPT: scatcor=yes, while scat=\"\"!", >> logfile)
	   print ("INTERUPT: scatcor=yes, while scat=\"\"!")
	   ok = no
	   return
	}
	if ( scatcor ) {
	   chkimg (cscat, "access", imtype="", verbose-)
	   i = 0 ; j = 0
	   dir (chkimg.impath//chkimg.imroot//"*"//chkimg.imtype, long-,
		ncols=1) | count | scan (i,j)
	   if ( i == 0 || ( i == 1 && j == 3 ) ) {
	      # dir returned "No files found"
	      ok = no
	      print ("ERROR: no scattered light images \""//chkimg.impath//chkimg.imroot//"*"//chkimg.imtype//"\" found!", >> logfile)
	      error (1, "no scattered light images \""//chkimg.impath//chkimg.imroot//"*"//chkimg.imtype//"\" found!")
	   } else {
	      scat = chkimg.impath//chkimg.imroot
	   }
	}

#	-- Memory available on host machine:
	if ( mx == INDEF ) {
	   # Try and query the host machine for the total amount of memory
	   # available. The following works on RedHat 7.x/8/9 systems, but
	   # is not portable on systems that aren't GNU based; in particular,
	   # 'free' may not be installed or its multi-line output format may
	   # differ...
	   # -- Redhat 9:
	   print ("!free|head -2|tail -1|awk '{printf(\"%d\\n\",(($2-65536)/("//str(nc)//"*"//str(nr)//"*4/1000)/2))}'") | cl | scan (mx)
	   # -- SunOS (sun4u) Ultra-60: try something like:
	   #print ("!top -b -d 1|head -4|tail -1 | awk '{printf(\"%d\\n\",(1024*substr($2,0,4)-65536)/("//str(nc)//"*"//str(nr)//"*4/1000)/2)}'") | cl | scan(mx)

	   # If the above failed, it hopefully failed miserably so that 'mx'
	   # remains equal to INDEF; if so, set 'mx' to minimum of 7, just
	   # in case the images we're dealing with are huge (7 4096^2 images
	   # would require just over 1 Gb of memory, but 7 is the minimum
	   # number for which image combination using 'avsigclip' behaves as
	   # expected)...
	   if ( mx == INDEF || mx < 7 ) mx = 7
	}

	# Update task parameters...
	isdir	= cisdir
	isfil	= cisfil
	biassec	= bsec
	trimsec	= tsec
	maxcomb	= mx
	ok	= yes

end
