procedure shcor48 (hdrcom, image, exptime, shutimg, shutfil, logfile,
		shutlog, verbose, first)

string	hdrcom	=""		{prompt="Header controled command"}
string	image	=""		{prompt="Image name"}
real	exptime	=INDEF		{prompt="Exposure time of image"}
string	shutimg	=""		{prompt="Shutter calibration image"}
string	shutfil =""		{prompt="Shutter effect coefficient file"}
string	logfile	=""		{prompt="Text logfile ccd48proc"}
string	shutlog	=""		{prompt="Text logfile shutter correction"}
bool	verbose	=yes		{prompt="Verbose messages?"}
bool	first	=no		{prompt="First image in list?"}
bool	ok	=no		{prompt="No errors encountered?"}

# @(#) subtask shcor48          Author: R.A. Jansen -- September 25 2003
# @(#) 
# @(#) subtask called from  'ccd48proc'  to correct short exposures  for
# @(#) the spatial variation  in actual exposure time  due to the finite
# @(#) time  it takes the shutter  to open and close.  Calibration image
# @(#) 'shut'  should be scaled  for an exposure time  of 1.0 sec.  Only
# @(#) images with exposure times shorter than 5 sec are corrected.
# @(#) The dependence of the logarithm of  the relative strength  of the
# @(#) shutter-effect on  the logarithm of the exposure time is supposed
# @(#) to have been fitted  using a simple polynomical  fitting function
# @(#)     FIT = SUM_i=0^n{a_i*(log10(exptime))**n}  with n<=7 (i.e., at
# @(#) most 8 coefficients),  and these coefficients to have been stored
# @(#) in 'shutfil'.  When n<7, the remaining coefficients should be set
# @(#) to 0.00.  The list of coefficients, one per line, may be preceded
# @(#) by an arbitrary number of commented lines (# as first character),
# @(#) but blank lines are not allowed. A second column of values (e.g.,
# @(#) errors on the coefficients) may be present, but is ignored.

begin
	string	cval, cimg, simg, sfil, cstr, sstr
	real	texp, val, a[8], f1, ft, fac
	int	nl, i, j
	bool	bok

	# Initialize variables...
	cval  = hdrcom
	cimg  = image
	texp  = exptime
	simg  = shutimg
	sfil  = shutfil
	bok   = no
	if ( shutlog == "" ) shutlog = "shutcor.log"

	# Re-initialize task parameter...
	ok = bok

	# Clean-up any relevant debris...
	if ( access("shcor.head") ) delete("shcor.head", yes, verify-)

	# If first image in list, open shutlog and start logging...
	if (first) {
	   sysinfo() ; gdate(utc=yes)
	   if ( access(shutlog) ) {
	      print ("\n## CCD48PROC (SHCOR48)  "//sysinfo.sysmsg//"  "//gdate.tdate//"\n", >> shutlog)
	   } else {
	      print ("## CCD48PROC (SHCOR48)  "//sysinfo.sysmsg//"  "//gdate.tdate//"\n", >> shutlog)
	   }
	}

	# Check whether image was processed already or exposure time is
	# longer than 5 sec...
	if ( cval == "COMPLETE" ) {
	   if (verbose) print ("    SHUTCOR:  image was processed already; no action taken")
	   print ("  SHUTCOR = \"COMPLETE\"; no action taken", >> logfile)
	   print (cimg//": SHUTCOR = \"COMPLETE\"; no action taken", >> shutlog)

	} else if ( cval == "OMIT" || texp > 5.0 ) {
	   if (cval == "OMIT" ) {
	      if (verbose) print ("    SHUTCOR = \"OMIT\"; no action taken")
	      print ("  SHUTCOR = \"OMIT\"; no action taken", >> logfile)
	      print (cimg//": SHUTCOR = \"OMIT\"; no action taken", >> shutlog)
	   } else {
	      if (verbose) printf("    SHUTCOR = \"OMIT\"; texp = %.1f > 5.0 sec\n", texp)
	      printf("  SHUTCOR = \"OMIT\"; texp = %.1f > 5.0 sec\n", texp, >> logfile)
	      printf(cimg//": SHUTCOR = \"OMIT\"; texp = %.1f > 5.0 sec\n", texp, >> shutlog)
	   }

	   # Add header keyword SHUTCOR...
	   gdate(utc=yes)
	   hedit (cimg, "shutcor", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "cc48date", "", add-, del+, ver-, show-, upd+)
	   imheader (cimg, longheader+, > "shcor.head")
	   printf("SHUTCOR = %-21s / Shutter effect correction flag\n", "'OMIT'", >> "shcor.head")
	   print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "shcor.head")
	   mkheader (cimg, "shcor.head", append-, verbose-)
	   delete ("shcor.head", yes, verify-)

	} else {
	   if (verbose) printf("    SHUTCOR ...")

	   # count number of lines in coefficients file...
	   iwc (sfil)
	   nl = iwc.nlines

	   # read in the array of shutter-effect coefficients; missing
	   # coefficients prompt a non-fatal warning, and are set to 0.00,
	   # despite the formal required format of 8 listed coefficients...
	   a[1] = 0.00 ; a[2] = 0.00 ; a[3] = 0.00 ; a[4] = 0.00
	   a[5] = 0.00 ; a[6] = 0.00 ; a[7] = 0.00 ; a[8] = 0.00
	   j = 0
	   for ( i = 1 ; i <= nl ; i = i + 1 ) {
	      rdlist (sfil, i)
	      if (!rdlist.ok) {
		 printf("\nSHCOR48: ERROR reading line %d from \""//sfil//"\"!\n", i)
		 printf("SHCOR48: ERROR reading line %d from \""//sfil//"\"!\n", i,>> logfile)
		 printf("SHCOR48: ERROR reading line %d from \""//sfil//"\"!\n", i, >> shutlog)
		 ok = no
		 return
              }
	      if ( substr(rdlist.cline,1,1) == "#" ) next

	      j = j + 1
	      if ( j > 8 ) {
		 print ("\nSHCOR48: ERROR reading coefficients from \""//sfil//"\": n_coef > 8!")
		 print ("SHCOR48: ERROR reading coefficients from \""//sfil//"\": n_coef > 8!", >> logfile)
		 print ("SHCOR48: ERROR reading coefficients from \""//sfil//"\": n_coef > 8!", >> shutlog)
		 ok = no
		 return
	      }

	      val = INDEF
	      cparse (rdlist.cline, delim=" ", nsel=1, ligno+, migno+, verb-)
	      printf("%.8f\n", real(cparse.cfield)) | scan(val)
	      if ( val == INDEF ) {
		 printf("\nSHCOR48: ERROR parsing coefficient %d from \""//sfil//"\"\n", j)
		 printf("SHCOR48: ERROR parsing coefficient %d from \""//sfil//"\"!\n", j, >> logfile)
		 printf("SHCOR48: ERROR parsing coefficient %d from \""//sfil//"\"!\n", j, >> shutlog)
		 ok = no
		 return
	      } else { a[j] = val }

	      #printf("SHCOR48 debug: a[%d] = %.8f\n", j, a[j])
	   }
	   if ( j < 8 ) {
	      printf("\nSHCOR48: WARNING %d coefficients read; remaining %d coeffs were set to 0\n", j, (8-j))
	      printf("SHCOR48: WARNING %d coefficients read; remaining %d coeffs were set to 0\n", j, (8-j), >> logfile)
	      printf("SHCOR48: WARNING %d coefficients read; remaining %d coeffs were set to 0\n", j, (8-j), >> shutlog)
	   }

	   # Compute relative strength of shutter effect at 1.0 sec and at
	   # 'texp' seconds from the polynomial fit:
	   f1 = 10**(a[1])
	   ft = 0.00
	   for ( i = 1 ; i <= 8 ; i = i + 1 ) {
	      ft = ft + a[i]*(log10(texp))**(i-1)
	   }
	   ft = 10**(ft)
	   fac = (ft-1.0)/(f1-1.0)
	   #printf("SHCOR48 debug: f(1.0)=%.8f, f(%3.1f)=%.8f, scalefac=%.4f\n", f1, texp, ft, fac)

	   # Scale the shutter-effect calibration image and correct the input
	   # image:
	   imcalc (cimg//","//simg, "tmpshcor.fits",
		"im1*(1.0-"//str(fac)//"*im2)", pixtype="real", nullval=0.,
		verbose=no)
	   imdelete (cimg, yes, verify-, default_acti+)
	   imrename ("tmpshcor.fits", cimg, verbose-)

	   # Add keywords SHUTCOR, SHCRIMAG, SHCRFMAX, and SHCRDATE...
	   gdate(utc=yes)
	   hedit (cimg, "shutcor", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "shcrimag", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "shcrfmax", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "shcrdate", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "cc48date", "", add-, del+, ver-, show-, upd+)
	   imheader (cimg, longheader+, > "shcor.head")
	   printf("SHUTCOR = %-21s / Shutter effect correction flag\n", "'COMPLETE'", >> "shcor.head")
	   if ( strlen(simg) <= 21 ) {
	      cstr = "'"//simg//"'"
	      printf("SHCRIMAG= %-21s / Name of shutter image\n", cstr, >> "shcor.head")
	   } else {
	      if ( strlen(simg) <= 44 ) {
		 cstr = "'"//simg//"'"
	      } else {
		 cstr = "'..."//substr(simg,(strlen(simg)-40),strlen(simg))//"'"
	      }
	      printf("SHCRIMAG= %-"//strlen(cstr)//"s / Name of shutter image\n", cstr, >> "shcor.head")
	   }
	   if ( strlen(sfil) <= 21 ) {
	      sstr = "'"//sfil//"'"
	      printf("SHCRCOEF= %-21s / Name of shutter coefficient file\n", sstr, >> "shcor.head")
	   } else {
	      if ( strlen(sfil) <= 33 ) {
		 sstr = "'"//sfil//"'"
	      } else {
		 sstr = "'..."//substr(sfil,(strlen(sfil)-29),strlen(sfil))//"'"
	      }
	      printf("SHCRCOEF= %-"//strlen(sstr)//"s / Name of shutter coefficient file\n", sstr, >> "shcor.head")
	   }
	   printf("SHCRFMAX= %20.4f  / Peak strength of shutter effect [%1s]\n", (100*(ft-1)), "%", >> "shcor.head")
	   print ("SHCRDATE= '"//gdate.fdate//"' / Shutter correction done", >> "shcor.head")
	   print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "shcor.head")
	   mkheader (cimg, "shcor.head", append-, verbose-)
	   delete ("shcor.head", yes, verify-)

	   if (verbose) printf("  done (peak=%.4f%1s)\n", (100*(ft-1)), "%")

	   # Log action to text log files...
	   print (cimg//":", >> shutlog)
	   printf("  SHUTCOR:  "//gdate.fdate//" Shutter correction using image "//cstr//"\n", >> shutlog)
	   printf("                                Coeff.file="//sstr//"\n", >> shutlog)
	   printf("                                Peak of shutter effect is %.4f%1s\n", (100*(ft-1)), "%", >> shutlog)
	   printf("  SHUTCOR:  "//gdate.fdate//" Shutter correction (%.4f%1s) using "//cstr//"\n", (100*(ft-1)), "%", >> logfile)

	}

	# Update task parameter...
	ok = yes

end
