procedure zcomb48 (hdrcom, inlist, nimg, imgtyp, statsec, drkkey, rdnoise,
		gain, maxval, maxcomb, logfile, comblog, verbose)

string	hdrcom	=""		{prompt="Header controled command"}
string	inlist	=""		{prompt="List with images to process"}
int	nimg	=INDEF		{prompt="Number of images in list"}
string	imgtyp	=""		{prompt="CCD image data type"}
string	statsec	=""		{prompt="Image section for statistics"}
string	drkkey	="darktime"	{prompt="Dark time header keyword"}
real	rdnoise	=INDEF		{prompt="CCD readout noise (e-)"}
real	gain	=INDEF		{prompt="CCD gain (e-/ADU)"}
real	maxval	=INDEF		{prompt="Maximum non-saturated/linear level"}
int	maxcomb	=7		{prompt="Max # of frames to combine at once"}
string	logfile	=""		{prompt="Text log file"}
string	comblog	=""		{prompt="Text log file image combination"}
bool	verbose	=yes		{prompt="Verbose messages?"}
bool	ok	=no		{prompt="No errors encountered?"}

# @(#) subtask zcomb48            Author: R.A. Jansen -- January 22 1996
# @(#) 
# @(#) subtask called from  'ccd48proc'  to combine the bias/dark frames
# @(#) after processing.  Parameter 'maxcomb' should match the machine's
# @(#) memory (minus that in use by the X server)  divided by the memory
# @(#) needed to store a single image (e.g. (256-64)/4.2 ~ 45  for 256Mb
# @(#) of RAM and 1024 x 1024 pixel  real valued images).  The 'rdnoise'
# @(#) and 'gain' parameters are at present dummy parameters,  since the
# @(#) images are combined using the  "avsigclip"  rather than "ccdclip"
# @(#) algorithm, but this may change in the future.
# @(#)
# @(#) Mar  6 2002 -- Headers are now updated  using  'artdata.mkheader'
# @(#)   rather than 'ccdhedit'. Keywords are now '[BIAS|DARK]COMB' with
# @(#)   value 'PERFORM','OMIT', or 'COMPLETE' and '[BS|DK]CBDATE' (FITS
# @(#)   compliant date + time string). RAJ.
# @(#) Aug 16 2003 -- Added  parameters  'hdrcom', 'nimg', 'drkkey'  and
# @(#)   'maxval'; completely rewrote this task. RAJ.
# @(#) Apr 21 2004 -- Fixed bug that split large image list into unequal
# @(#)   sized batches; fixed bug of badly formatted header entry. Added
# @(#)   check whether input image list contains at least 2 images. RAJ.
# @(#) May 11 2004 -- Fixed format error in BSCBDATE string. RAJ.
# @(#) Jul  5 2004 -- Along with the combined image a sigma image is now
# @(#)   output as well. RAJ.

begin
	string	cval, clis, dtyp, ctyp, csec, ckey, cnam, root
	string	scle, zrof, wght, cstr, dstr, cout, sout
	struct	line
	real	ron, eadu, mxval, mnval
	int	ntot, nmax, i, j, k, m
	bool	bok, new

	# Initialize variables...
	cval  = hdrcom
	clis  = inlist
	if ( nimg == INDEF || nimg < 0 ) {
	   if ( nimg == INDEF ) print ("ERROR: ZCOMB48 called with nimg=INDEF!")
	   if ( nimg < 0 ) print ("ERROR: ZCOMB48 called with nimg < 0!")
	   ok = no
	   return
	} else {
	   ntot  = nimg
	}
	dtyp  = imgtyp
	csec  = statsec
	ckey  = drkkey
	ron   = rdnoise
	eadu  = gain
	mxval = maxval
	mnval = INDEF
	if ( mxval != INDEF ) mnval = -0.01*mxval
	nmax  = maxcomb
	bok   = no
	new   = ( release >= "2.12" )
	ctyp  = "."//envget("imtype")

	# Check type of image data and (re)initialize task parameter...
	if ( dtyp == "ZERO" ) { dtyp = "BIAS" }
	if ( dtyp != "BIAS" && dtyp != "DARK" ) {
	   print ("ERROR: ZCOMB48 called for incorrect image type "//dtyp//"!")
	   ok = no
	   return
	} else {
	   ok = bok
	}

	# Define image combination parameters...
	if ( dtyp == "BIAS" ) {
	   root  = "bias" 
	   scle  = "none"
	   zrof  = "mode"
	   wght  = "none"
	   cstr  = "Bias"
	   dstr  = "BS"
	} else {
	   root  = "dark" 
	   scle  = "exposure"
	   zrof  = "none"
	   wght  = "exposure"
	   cstr  = "Dark"
	   dstr  = "DK"
	}

	# Clean-up any relevant debris...
	if ( access("zcomb.head") ) delete ("zcomb.head", yes, verify-)

	# Open comblog and start logging...
	sysinfo() ; gdate(utc=yes)
	if ( access(comblog) ) {
	   print ("\n## CCD48PROC (ZCOMB48)  "//sysinfo.sysmsg//"  "//gdate.sdate//"\n", >> comblog)
	} else {
	   print ("## CCD48PROC (ZCOMB48)  "//sysinfo.sysmsg//"  "//gdate.sdate//"\n", >> comblog)
	}
	if (verbose) print ("CCD48PROC Input=\"@"//clis//"\":")
	print ("\nCCD48PROC Input=\"@"//clis//"\":", >> logfile)
	print ("\nCCD48PROC Input=\"@"//clis//"\":\n", >> comblog)

	# Even if 'hdrcom' reads 'COMPLETE', the combined image may have
	# been accidentally deleted. Since combination does not affect the
	# individual images, in this instance, 'COMPLETE' will be ignored.
	if ( cval == "OMIT" ) {
	   if (verbose) print ("    "//dtyp//"COMB= \"OMIT\"; no action taken")
	   print ("  "//dtyp//"COMB= \"OMIT\"; no action taken", >> logfile)
	   print ("  "//dtyp//"COMB= \"OMIT\"; no action taken\n", >> comblog)

	   # Add header keyword 'dtyp'//COMB...
	   gdate(utc=yes)
	   list = clis
	   i = 0
	   while ( fscan(list,line) != EOF ) {
	      cnam = line
	      i = i + 1
	      if ( (i%5) == 0 ) gdate(utc=yes)
	      hedit (cnam, "ncombine", "", add-, del+, ver-, show-, upd+)
	      hedit (cnam, dtyp//"COMB", "", add-, del+, ver-, show-, upd+)
	      hedit (cnam, dstr//"CBDATE", "", add-, del+, ver-, show-, upd+)
	      hedit (cnam, "imcmb*", "", add-, del+, ver-, show-, upd+)
	      hedit (cnam, "cc48date", "", add-, del+, ver-, show-, upd+)
	      imheader (cnam, longheader+, > "zcomb.head")
	      sleep(1)
	      printf(dtyp//"COMB= %-21s / "//cstr//" frame combination flag\n", "'OMIT'", >> "zcomb.head")
	      printf("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "zcomb.head")
	      mkheader (cnam, "zcomb.head", append-, verbose-)
	      delete ("zcomb.head", yes, verify-)
	   }
	   list = ""

	} else {
	   # Check whether we can combine all images at once or need to
	   # combine them in batches...
	   if ( ntot < nmax ) {
	      # check that there are at least 2 images to combine...
	      if ( ntot < 1 ) {
		 if (verbose) print ("    "//dtyp//"COMB: \""//clis//"\" is empty; no action taken")
		 print ("    "//dtyp//"COMB: \""//clis//"\" is empty; no action taken", >> logfile)
		 print ("    "//dtyp//"COMB: \""//clis//"\" is empty; no action taken\n", >> comblog)
		 ok = no
		 return
	      } else if ( ntot < 2 ) {
		 if (verbose) print ("    "//dtyp//"COMB: only 1 image in \""//clis//"\"; no action taken")
		 print ("    "//dtyp//"COMB: only 1 image in \""//clis//"\"; no action taken", >> logfile)
		 print ("    "//dtyp//"COMB: only 1 image in \""//clis//"\"; no action taken\n", >> comblog)
		 ok = yes
		 return
	      }

	      # Construct the output image name...
	      cout = dtyp//ctyp

	      # Check whether the output frame exists already...
	      chkimg (cout, "noaccess", imtype="", verbose+)
	      if (!chkimg.ok) { return }
	      cout = chkimg.impath//chkimg.imroot//chkimg.imtype
	      sout = chkimg.impath//chkimg.imroot//"_e"//chkimg.imtype

	      if (verbose) print ("    "//dtyp//"COMB... result=\""//cout//"\"")

	      # Combine all images in the input list...
	      if ( new ) {
		 imcombine ("@"//clis, output=cout, headers="", bpmasks="",
			rejmasks="", nrejmasks="", expmasks="", sigma=sout,
			logfile=comblog, combine="average", reject="avsigclip",
			project=no, outtype="real", outlimits="",
			offsets="none", masktype="none", maskvalue=0., blank=0.,
			scale=scle, zero=zrof, weight=wght, statsec=csec,
			expname=ckey, lthreshold=mnval, hthreshold=mxval,
			nlow=0, nhigh=1, nkeep=1, mclip=yes, lsigma=3.,
			hsigma=3., rdnoise=str(ron), gain=str(eadu),
			snoise="0.", sigscale=0.1, pclip=-0.5, grow=0.5)
	      } else {
		 imcombine ("@"//clis, output=cout, rejmask="", plfile="",
			sigma=sout, logfile=comblog, combine="average",
			reject="avsigclip", project=no, outtype="real",
			offsets="none", masktype="none", maskvalue=0., blank=0.,
			scale=scle, zero=zrof, weight=wght, statsec=csec,
			expname=ckey, lthreshold=mnval, hthreshold=mxval,
			nlow=0, nhigh=1, nkeep=1, mclip=yes, lsigma=3.,
			hsigma=3., rdnoise=str(ron), gain=str(eadu),
			snoise="0.", sigscale=0.1, pclip=-0.5, grow=0.5)
	      }

	      # Add keywords [IMGTYP]COMB and [BS|DK]CBDATE to image header...
	      gdate(utc=yes)
	      hedit (cout, "ncombine", "", add-, del+, ver-, show-, upd+)
	      hedit (cout, dtyp//"COMB", "", add-, del+, ver-, show-, upd+)
	      hedit (cout, dstr//"CBDATE", "", add-, del+, ver-, show-, upd+)
	      hedit (cout, "imcmb*", "", add-, del+, ver-, show-, upd+)
	      hedit (cout, "cc48date", "", add-, del+, ver-, show-, upd+)
	      imheader (cout, longheader+, > "zcomb.head")
	      sleep(1)
	      printf(dtyp//"COMB= %-21s / "//cstr//" frame combination flag\n", "'COMPLETE'", >> "zcomb.head")
	      printf("NCOMBINE= %20d  / Number of "//root//" frames combined\n", ntot, >> "zcomb.head")
	      print (dstr//"CBDATE= '"//gdate.fdate//"' / Combination of "//root//" frames @"//clis//" done", >> "zcomb.head")
	      print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "zcomb.head")
	      mkheader (cout, "zcomb.head", append-, verbose-)
	      mkheader (sout, "zcomb.head", append-, verbose-)
	      delete ("zcomb.head", yes, verify-)

	      # Log action to text log files...
	      print ("  "//dtyp//"COMB: "//gdate.fdate//" Combined "//dtyp//" frames (all at once) -> \""//cout//"\"", >> logfile)
	      print ("  "//dtyp//"COMB: "//gdate.fdate//" Combined "//dtyp//" frames (all at once) -> \""//cout//"\"\n", >> comblog)

	      # Add header keyword 'dtyp'//COMB to images in list...
	      list = clis
	      while ( fscan(list,line) != EOF ) {
		 cnam = line
		 hedit (cnam, "ncombine", "", add-, del+, ver-, show-, upd+)
		 hedit (cnam, dtyp//"COMB", "", add-, del+, ver-, show-, upd+)
		 hedit (cnam, dstr//"CBDATE", "", add-, del+, ver-, show-, upd+)
		 hedit (cnam, "imcmb*", "", add-, del+, ver-, show-, upd+)
		 hedit (cnam, "cc48date", "", add-, del+, ver-, show-, upd+)
		 imheader (cnam, longheader+, > "zcomb.head")
		 sleep(1)
		 printf(dtyp//"COMB= %-21s / "//cstr//" frame combination flag\n", "'COMPLETE'", >> "zcomb.head")
		 print (dstr//"CBDATE= '"//gdate.fdate//"' / Combination of "//root//" frames @"//clis//" done", >> "zcomb.head")
		 printf("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "zcomb.head")
		 mkheader (cnam, "zcomb.head", append-, verbose-)
		 delete ("zcomb.head", yes, verify-)
	      }
	      list = ""

	   } else {
	      # Number of batches to subdivide image list into...
	      k = nint(1.0*ntot/nmax)

	      # Write the k lists (batches) of images to combine...
	      m = 1
	      for ( i = 1 ; i <= k ; i = i + 1 ) {
		 cnam = "zcomb"//str(i)//".lis"
		 if ( access(cnam) ) delete (cnam, yes, verify-)

		 for ( j = m ; j <= nint(1.0*i*ntot/k) ; j = j + 1 ) {
		    rdlist (clis, j)
		    if (!rdlist.ok) {
			print ("ZCOMB48: ERROR reading line "//str(j)//" from \""//clis//"\"!", >> logfile)
			print ("ZCOMB48: ERROR reading line "//str(j)//" from \""//clis//"\"!")
			ok = no
			return
		    } else {
			print (rdlist.cline, >> cnam)
		    }
		 }

		 m = nint(1.0*i*ntot/k) + 1
	      }

	      # Now combine the images within each of the batches...
	      if ( access("zcomb.lis") ) delete ("zcomb.lis", yes, verify-)
	      for ( i = 1 ; i <= k ; i = i + 1 ) {
		 cout = dtyp//str(i)//ctyp
		 print (cout, >> "zcomb.lis")
		 cnam = "zcomb"//str(i)//".lis"
		 iwc (cnam)

		 # Check whether the output frame exists already...
		 chkimg (cout, "noaccess", imtype="", verbose+)
		 if (!chkimg.ok) { return }
		 cout = chkimg.impath//chkimg.imroot//chkimg.imtype

		 if (verbose) printf("    "//dtyp//"COMB... batch %d -> \""//cout//"\" (NCOMBINE=%d)\n", i, iwc.nlines)

		 if ( new ) {
		    imcombine ("@"//cnam, output=cout, headers="", bpmasks="",
			rejmasks="", nrejmasks="", expmasks="", sigma="",
			logfile=comblog, combine="average", reject="avsigclip",
			project=no, outtype="real", outlimits="",
			offsets="none", masktype="none", maskvalue=0., blank=0.,
			scale=scle, zero=zrof, weight=wght, statsec=csec,
			expname=ckey, lthreshold=mnval, hthreshold=mxval,
			nlow=0, nhigh=1, nkeep=1, mclip=yes, lsigma=3.,
			hsigma=3., rdnoise=str(ron), gain=str(eadu),
			snoise="0.", sigscale=0.1, pclip=-0.5, grow=0.5)
		 } else {
		    imcombine ("@"//cnam, output=cout, rejmask="", plfile="",
			sigma="", logfile=comblog, combine="average",
			reject="avsigclip", project=no, outtype="real",
			offsets="none", masktype="none", maskvalue=0., blank=0.,
			scale=scle, zero=zrof, weight=wght, statsec=csec,
			expname=ckey, lthreshold=mnval, hthreshold=mxval,
			nlow=0, nhigh=1, nkeep=1, mclip=yes, lsigma=3.,
			hsigma=3., rdnoise=str(ron), gain=str(eadu),
			snoise="0.", sigscale=0.1, pclip=-0.5, grow=0.5)
		 }

		 # Add keywords [IMGTYP]COMB and [BS|DK]CBDATE to image header...
		 gdate(utc=yes)
		 hedit (cout, "ncombine", "", add-, del+, ver-, show-, upd+)
		 hedit (cout, dtyp//"COMB", "", add-, del+, ver-, show-, upd+)
		 hedit (cout, dstr//"CBDATE", "", add-, del+, ver-, show-, upd+)
		 hedit (cout, "imcmb*", "", add-, del+, ver-, show-, upd+)
		 hedit (cout, "cc48date", "", add-, del+, ver-, show-, upd+)
		 imheader (cout, longheader+, > "zcomb.head")
		 sleep(1)
		 printf(dtyp//"COMB= %-21s / "//cstr//" frame combination flag\n", "'COMPLETE'", >> "zcomb.head")
		 printf("NCOMBINE= %20d  / Number of "//root//" frames combined\n", iwc.nlines, >> "zcomb.head")
		 print (dstr//"CBDATE= '"//gdate.fdate//"' / Combination of "//root//" frames @"//cnam//" done", >> "zcomb.head")
		 print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "zcomb.head")
		 mkheader (cout, "zcomb.head", append-, verbose-)
		 delete ("zcomb.head,"//cnam, yes, verify-)

		 # Log action to text log files...
		 if ( i == 1 ) {
		    printf("  "//dtyp//"COMB: "//gdate.fdate//" Combined "//dtyp//" frames (batch %d) -> \""//cout//"\"\n\n", i, >> comblog)
		 } else {
		    printf("\n  "//dtyp//"COMB: "//gdate.fdate//" Combined "//dtyp//" frames (batch %d) -> \""//cout//"\"\n\n", i, >> comblog)
		 }
		 printf("  "//dtyp//"COMB: "//gdate.fdate//" Combined "//dtyp//" frames (batch %d) -> \""//cout//"\"\n", i, >> logfile)

	      }

	      # Finally, combine the frames combined per batch in a single
	      # output frame...
	      cout = dtyp//ctyp

	      # Check whether the output frame exists already...
	      chkimg (cout, "noaccess", imtype="", verbose+)
	      if (!chkimg.ok) { return }
	      cout = chkimg.impath//chkimg.imroot//chkimg.imtype
	      sout = chkimg.impath//chkimg.imroot//"_e"//chkimg.imtype

	      if (verbose) {
		 printf("    "//dtyp//"COMB... batches 1--%d -> \""//cout//"\" (NCOMBINE=%d)\n", k, ntot)
	      }
	      if ( new ) {
		 imcombine ("@zcomb.lis", output=cout, headers="", bpmasks="",
			rejmasks="", nrejmasks="", expmasks="", sigma=sout,
			logfile=comblog, combine="average", reject="avsigclip",
			project=no, outtype="real", outlimits="",
			offsets="none", masktype="none", maskvalue=0., blank=0.,
			scale=scle, zero=zrof, weight=wght, statsec=csec,
			expname=ckey, lthreshold=mnval, hthreshold=mxval,
			nlow=0, nhigh=1, nkeep=1, mclip=yes, lsigma=3.,
			hsigma=3., rdnoise=str(ron), gain=str(eadu),
			snoise="0.", sigscale=0.1, pclip=-0.5, grow=0.5)
	      } else {
		 imcombine ("@zcomb.lis", output=cout, rejmask="", plfile="",
			sigma=sout, logfile=comblog, combine="average",
			reject="avsigclip", project=no, outtype="real",
			offsets="none", masktype="none", maskvalue=0., blank=0.,
			scale=scle, zero=zrof, weight=wght, statsec=csec,
			expname=ckey, lthreshold=mnval, hthreshold=mxval,
			nlow=0, nhigh=1, nkeep=1, mclip=yes, lsigma=3.,
			hsigma=3., rdnoise=str(ron), gain=str(eadu),
			snoise="0.", sigscale=0.1, pclip=-0.5, grow=0.5)
	      }

	      # Add keywords [IMGTYP]COMB and [BS|DK]CBDATE to image header...
	      gdate(utc=yes)
	      hedit (cout, "ncombine", "", add-, del+, ver-, show-, upd+)
	      hedit (cout, dtyp//"COMB", "", add-, del+, ver-, show-, upd+)
	      hedit (cout, dstr//"CBDATE", "", add-, del+, ver-, show-, upd+)
	      hedit (cout, "imcmb*", "", add-, del+, ver-, show-, upd+)
	      hedit (cout, "cc48date", "", add-, del+, ver-, show-, upd+)
	      imheader (cout, longheader+, > "zcomb.head")
	      sleep(1)
	      printf(dtyp//"COMB= %-21s / "//cstr//" frame combination flag\n", "'COMPLETE'", >> "zcomb.head")
	      printf("NCOMBINE= %20d  / Number of "//root//" frames combined\n", ntot, >> "zcomb.head")
	      print (dstr//"CBDATE= '"//gdate.fdate//"' / Combination of "//root//" frames @"//clis//" done", >> "zcomb.head")
	      print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "zcomb.head")
	      mkheader (cout, "zcomb.head", append-, verbose-)
	      mkheader (sout, "zcomb.head", append-, verbose-)
	      delete ("zcomb.head,zcomb.lis", yes, verify-)

	      # Log action to text log files...
	      printf("  "//dtyp//"COMB: "//gdate.fdate//" Combined batches 1--%d -> \""//cout//"\"\n", k, >> logfile)
	      printf("\n  "//dtyp//"COMB: "//gdate.fdate//" Combined batches 1--%d --> \""//cout//"\"\n\n", k, >> comblog)

	   }

	}

	# Update task parameter...
	ok = yes

end
