procedure zrcor48 (hdrcom, image, zero, logfile, zerolog, verbose, first)

string	hdrcom	=""		{prompt="Header controled command"}
string	image	=""		{prompt="Image name"}
string	zero	=""		{prompt="Zero level calibration image"}
string	logfile	=""		{prompt="Text logfile ccd48proc"}
string	zerolog	=""		{prompt="Text logfile biassubtraction"}
bool	verbose	=yes		{prompt="Verbose messages?"}
bool	first	=no		{prompt="First image in list?"}

# @(#) subtask zrcor48            Author: R.A. Jansen -- January 17 1996
# @(#) 
# @(#) subtask called from  'ccd48proc'  to correct for structure in the
# @(#) bias level using a zero exposure calibration frame  ('masterbias'
# @(#) frame). 
# @(#)
# @(#) Aug 26 2002 -- Headers are now updated  using  'artdata.mkheader'
# @(#)   rather than 'ccdhedit'.  Keywords are now 'ZEROCOR' (with value
# @(#)   'PERFORM','OMIT', or 'COMPLETE'), 'ZRCRIMAG' with image name as
# @(#)   its value and 'ZRCRDATE' (FITS Y2K compliant date+time string).
# @(#) Aug 13 2003 -- Added 'hdrcom' to support header keyword controled
# @(#)   data reduction (a la 'calstis') and added 'verbose' switch. RAJ

begin
	string	cval, cimg, zimg, ctyp, logstr, cstr

	# Initialize variables...
	cval  = hdrcom
	cimg  = image
	zimg  = zero
	ctyp  = "."//envget("imtype")
	cimg  = cimg//ctyp
	if ( zerolog == "" ) zerolog = "biasproc.log"

	# Clean-up any relevant debris...
	if ( access("zrcor.head") ) delete("zrcor.head", yes, verify-)

	# If first image in list, open zerolog and start logging...
	if (first) {
	   sysinfo() ; gdate(utc=yes)
	   if ( access(zerolog) ) {
	      print ("\n## CCD48PROC (ZRCOR48)  "//sysinfo.sysmsg//"  "//gdate.tdate//"\n", >> zerolog)
	   } else {
	      print ("## CCD48PROC (ZRCOR48)  "//sysinfo.sysmsg//"  "//gdate.tdate//"\n", > zerolog)
	   }
	}

	# Check whether image was processed already...
	if ( cval == "COMPLETE" ) {
	   if (verbose) print ("    ZEROCOR:  image was processed already; no action taken")
	   print ("  ZEROCOR = \"COMPLETE\"; no action taken", >> logfile)
	   print (cimg//": ZEROCOR = \"COMPLETE\"; no action taken", >> zerolog)

	} else if ( cval == "OMIT" ) {
	   if (verbose) print ("    ZEROCOR = \"OMIT\"; no action taken")
	   print ("  ZEROCOR = \"OMIT\"; no action taken", >> logfile)
	   print (cimg//": ZEROCOR = \"OMIT\"; no action taken", >> zerolog)

	   # Add header keyword ZEROCOR...
	   gdate(utc=yes)
	   hedit (cimg, "zerocor", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "cc48date", "", add-, del+, ver-, show-, upd+)
	   imheader (cimg, longheader+, > "zrcor.head")
	   printf("ZEROCOR = %-21s / Zero image correction flag\n", "'OMIT'", >> "zrcor.head")
	   print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "zrcor.head")
	   mkheader (cimg, "zrcor.head", append-, verbose-)
	   delete ("zrcor.head", yes, verify-)

	} else {
	   if (verbose) printf("    ZEROCOR ...  ")
	   printf(cimg//":\n", >> zerolog)

	   # Subtract zero image from input image...
	   imarith (cimg, "-", zimg, cimg, title="", divzero=0.,
		hparams="", pixtype="real", calctype="real", verbose+,
		noact-) | match ("IMARITH:", "-", stop+, >> zerolog)

	   # Add keywords ZEROCOR, ZRCRIMAG and ZRCRDATE to image header...
	   gdate(utc=yes)
	   hedit (cimg, "zerocor", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "zrcrimag", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "zrcrdate", "", add-, del+, ver-, show-, upd+)
	   hedit (cimg, "cc48date", "", add-, del+, ver-, show-, upd+)
	   imheader (cimg, longheader+, > "zrcor.head")
	   printf("ZEROCOR = %-21s / Zero image correction flag\n", "'COMPLETE'", >> "zrcor.head")
	   if ( strlen(zimg) <= 21 ) {
	      cstr = "'"//zimg//"'"
	      printf("ZRCRIMAG= %-21s / Name of zero image\n", cstr, >> "zrcor.head")
	   } else {
	      if ( strlen(zimg) <= 47 ) {
		 cstr = "'"//zimg//"'"
	      } else {
		 cstr = "'..."//substr(zimg,(strlen(zimg)-43),strlen(zimg))//"'"
	      }
	      printf("ZRCRIMAG= %-"//strlen(cstr)//"s / Name of zero image\n", cstr, >> "zrcor.head")
	   }
	   print ("ZRCRDATE= '"//gdate.fdate//"' / Zero correction using image done", >> "zrcor.head")
	   print ("CC48DATE= '"//gdate.fdate//"' / Primary frame processing done", >> "zrcor.head")
	   mkheader (cimg, "zrcor.head", append-, verbose-)
	   delete ("zrcor.head", yes, verify-)

	   if (verbose) printf("done\n")

	   # Log action to text log files...
	   print ("ZEROCOR:  "//gdate.fdate//" Zero correction using image "//cstr, >> zerolog)
	   print ("  ZEROCOR:  "//gdate.fdate//" Zero correction using image "//cstr, >> logfile)

	}

end
